#!/usr/bin/env python
r"""
Raman Flow with BSE and frozen phonon
=====================================

This script shows how to perform a Raman calculation with
excitonic effects included with the BSE formalism.
"""
from __future__ import division, print_function, unicode_literals, absolute_import

import sys
import os
import numpy as np
import abipy.abilab as abilab
import abipy.data as abidata
from abipy import flowtk


def build_flow(options):
    # Working directory (default is the name of the script with '.py' removed and "run_" replaced by "flow_")
    if not options.workdir:
        options.workdir = os.path.basename(__file__).replace(".py", "").replace("run_","flow_")

    flow = flowtk.Flow(options.workdir, manager=options.manager)

    pseudos = abidata.pseudos("14si.pspnc")

    # Get the unperturbed structure.
    base_structure = abidata.structure_from_ucell("Si")

    etas = [-.001, 0, +.001]
    ph_displ = np.reshape(np.zeros(3*len(base_structure)), (-1,3))
    ph_displ[0,:] = [+1, 0, 0]
    ph_displ[1,:] = [-1, 0, 0]

    # Build new structures by displacing atoms according to the phonon displacement
    # ph_displ (in cartesian coordinates). The Displacement is normalized so that
    # the maximum atomic diplacement is 1 Angstrom and then multiplied by eta.
    modifier = abilab.StructureModifier(base_structure)

    displaced_structures = modifier.displace(ph_displ, etas, frac_coords=False)

    # Generate the different shifts to average
    ndiv = 2
    shift1D = np.arange(1,2*ndiv+1,2)/(2*ndiv)
    all_shifts = [[x,y,z] for x in shift1D for y in shift1D for z in shift1D]
    all_shifts = [[0, 0, 0]]

    for structure, eta in zip(displaced_structures, etas):
        for shift in all_shifts:
            flow.register_work(raman_work(structure, pseudos, shift))

    return flow


def raman_work(structure, pseudos, shiftk, paral_kgb=1):
    # Generate 3 different input files for computing optical properties with BSE.

    # Global variables
    global_vars = dict(
        ecut=8,
        istwfk="*1",
        chksymbreak=0,
        #nstep=4,
        nstep=10,
        paral_kgb=paral_kgb,
    )

    # GS run
    scf_inp = abilab.AbinitInput(structure, pseudos=pseudos)
    scf_inp.set_vars(global_vars)
    scf_inp.set_kmesh(ngkpt=[2, 2, 2], shiftk=shiftk)
    scf_inp["tolvrs"] = 1e-6

    # NSCF run
    nscf_inp = abilab.AbinitInput(structure, pseudos=pseudos)
    nscf_inp.set_vars(global_vars)
    nscf_inp.set_kmesh(ngkpt=[2, 2, 2], shiftk=shiftk)

    nscf_inp.set_vars(
        tolwfr=1e-8,
        nband=12,
        nbdbuf=4,
        iscf=-2,
    )

    # BSE run with Model dielectric function and Haydock (only resonant + W + v)
    # Note that SCR file is not needed here
    bse_inp = abilab.AbinitInput(structure, pseudos=pseudos)
    bse_inp.set_vars(global_vars)
    bse_inp.set_kmesh(ngkpt=[2, 2, 2], shiftk=shiftk)

    bse_inp.set_vars(
        optdriver=99,
        ecutwfn=global_vars["ecut"],
        ecuteps=3,
        inclvkb=2,
        bs_algorithm=2,       # Haydock
        bs_haydock_niter=4,   # No. of iterations for Haydock
        bs_exchange_term=1,
        bs_coulomb_term=21,   # Use model W and full W_GG.
        mdf_epsinf=12.0,
        bs_calctype=1,        # Use KS energies and orbitals to construct L0
        mbpt_sciss="0.8 eV",
        bs_coupling=0,
        bs_loband=2,
        nband=6,
        #bs_freq_mesh="0 10 0.1 eV",
        bs_hayd_term=0,      # No terminator
    )

    # Build the work representing a BSE run with model dielectric function.
    return flowtk.BseMdfWork(scf_inp, nscf_inp, bse_inp)


# This block generates the thumbnails in the Abipy gallery.
# You can safely REMOVE this part if you are using this script for production runs.
if os.getenv("READTHEDOCS", False):
    __name__ = None
    import tempfile
    options = flowtk.build_flow_main_parser().parse_args(["-w", tempfile.mkdtemp()])
    #build_flow(options).plot_networkx(with_edge_labels=True, tight_layout=True)
    build_flow(options).graphviz_imshow()


@flowtk.flow_main
def main(options):
    """
    This is our main function that will be invoked by the script.
    flow_main is a decorator implementing the command line interface.
    Command line args are stored in `options`.
    """
    return build_flow(options)


if __name__ == "__main__":
    sys.exit(main())


############################################################################
#
# Run the script with:
#
#     run_raman_bse.py -s
#
# then use:
#
#    abirun.py flow_raman_bse robot MDF
#
# to analyze all the macroscopic dielectric functions produced by the Flow with the robot.
# Inside the ipython terminal type:
#
# .. code-block:: ipython
#
#   [1] %matplotlib
#   [2] robot.plot()
#
# to compare the real and imaginary part of the macroscopic dielectric function for the different displacements.
#
# .. image:: https://github.com/abinit/abipy_assets/blob/master/run_raman_bse.png?raw=true
#    :alt: Band structure of Si in the IBZ and along a k-path
#



#!/usr/bin/env python

import abipy.data as abidata

from abipy.gui.wxapps import wxapp_structure_converter

filename = abidata.ref_file("si_nscf_GSR.nc")

app = wxapp_structure_converter(filename)
app.MainLoop()



from __future__ import print_function, division, unicode_literals, absolute_import

import os
import wx
import abipy.gui.awx as awx

from wx.py.shell import Shell
from monty.string import marquee
from abipy.abilab import abiopen
from abipy.gui import mixins as mix
from abipy.gui.kpoints import KpointsPanel
from abipy.gui.baseviewer import MultiViewerFrame


class GsrViewerFrame(MultiViewerFrame, mix.Has_Structure, mix.Has_MultipleEbands, mix.Has_Tools, mix.Has_NetcdfFiles):
    VERSION = "0.1"

    HELP_MSG = """Quick help:

 Kpoint list:

     Right-Click:  display popup menu with choices.

Also, these key bindings can be used
(For Mac OSX, replace 'Ctrl' with 'Apple'):

  Ctrl-Q:     quit
"""

    @property
    def codename(self):
        return "GsrViewer"

    @property
    def active_gsr(self):
        """The active GSR file i.e. the GSR associated to the active tab."""
        return self.active_tab.gsr

    @property
    def structure(self):
        """`Structure` associated to the active tab."""
        return self.active_gsr.structure

    @property
    def ebands(self):
        """`ElectronBands` associated to the active tab."""
        return self.active_gsr.ebands

    @property
    def ebands_list(self):
        """List of `ElectronBands`."""
        ebands_list = []
        for page in range(self.notebook.GetPageCount()):
            tab = self.notebook.GetPage(page)
            ebands_list.append(tab.gsr.ebands)
        return ebands_list

    @property
    def ebands_filepaths(self):
        """
        Return a list with the absolute paths of the files
        from which the `ElectronBands` have been read.
        """
        paths = []
        for page in range(self.notebook.GetPageCount()):
            tab = self.notebook.GetPage(page)
            paths.append(tab.gsr.filepath)
        return paths

    @property
    def nc_filepaths(self):
        """String with the absolute path of the netcdf file."""
        paths = []
        for page in range(self.notebook.GetPageCount()):
            tab = self.notebook.GetPage(page)
            paths.append(tab.gsr.filepath)
        return paths

    def makeMenu(self):
        """Creates the main menu."""
        # Base menu
        menu_bar = super(GsrViewerFrame, self).makeMenu()

        # Add Mixin menus.
        menu_bar.Append(self.CreateStructureMenu(), "Structure")
        menu_bar.Append(self.CreateEbandsMenu(), "Ebands")
        menu_bar.Append(self.CreateToolsMenu(), "Tools")
        menu_bar.Append(self.CreateNetcdfMenu(), "Netcdf")

        # Help menu
        help_menu = self.makeHelpMenu()
        menu_bar.Append(help_menu, "Help")

        self.SetMenuBar(menu_bar)

    def makeToolBar(self):
        """Creates the toolbar."""
        self.toolbar = toolbar = self.CreateToolBar()
        toolbar.SetToolBitmapSize(wx.Size(48, 48))

        def bitmap(path):
            return wx.Bitmap(awx.path_img(path))

        artBmp = wx.ArtProvider.GetBitmap
        toolbar.AddSimpleTool(wx.ID_OPEN, artBmp(wx.ART_FILE_OPEN, wx.ART_TOOLBAR), "Open")
        #toolbar.AddSeparator()
        toolbar.Realize()

    def addFileTab(self, parent, filepath):
        """Read data from filepath and create a new notebook tab."""
        gsr = abiopen(filepath)
        tab = GsrFileTab(self.notebook, gsr)
        self.notebook.AddPage(tab, os.path.basename(filepath))


class GsrFileTab(wx.Panel):
    """Tab showing information on a single GSR file."""
    def __init__(self, parent, gsr, **kwargs):
        """
        Args:
            parent:
                parent window.
            gsr:
                `GsrFile` instance.
        """
        super(GsrFileTab, self).__init__(parent, -1, **kwargs)
        self.gsr = gsr

        splitter = wx.SplitterWindow(self, id=-1, style=wx.SP_3D)
        splitter.SetSashGravity(0.95)

        self.kpoints_panel = KpointsPanel(splitter, gsr.structure, gsr.kpoints)

        # Add Python shell
        msg = "GSR_File object is accessible via the gsr variable. Use gsr.<TAB> to access the list of methods."
        msg = marquee(msg, width=len(msg) + 8, mark="#")
        msg = "#"*len(msg) + "\n" + msg + "\n" + "#"*len(msg) + "\n"

        pyshell = Shell(splitter, introText=msg, locals={"gsr": self.gsr})
        splitter.SplitHorizontally(self.kpoints_panel, pyshell)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(splitter, 1, wx.EXPAND, 5)
        self.SetSizerAndFit(sizer)

    @property
    def statusbar(self):
        return self.viewer_frame.statusbar

    def GetVisualizer(self):
        """Returns a string with the visualizer selected by the user."""
        return self.viewer_frame.GetVisualizer()

    def _visualize_skb(self, spin, kpoint, band):
        """Calls the visualizer to visualize the specified wavefunction."""
        # To make the Gui responsive one can use the approach described in
        # http://wiki.wxpython.org/LongRunningTasks
        appname = self.GetVisualizer()
        if appname == "None": return

        self.statusbar.PushStatusText("Visualizing wavefunction (spin=%d, kpoint=%s, band=%d)" % (spin, kpoint, band))
        try:
            visu = self.wfk.visualize_ur2(spin, kpoint, band, appname=appname)
            thread = awx.WorkerThread(self, target=visu)
            thread.start()

        except:
            awx.showErrorMessage(self)

    @property
    def viewer_frame(self):
        """The parent frame `GsrViewerFrame`."""
        try:
            return self._viewer_frame

        except AttributeError:
            self._viewer_frame = self.getParentWithType(GsrViewerFrame)
            return self._viewer_frame


class GsrViewerApp(awx.App):
    pass


def wxapp_gsrviewer(gsr_filepaths):
    """Standalone application."""
    app = GsrViewerApp()
    frame = GsrViewerFrame(None, filepaths=gsr_filepaths)
    app.SetTopWindow(frame)
    frame.Show()
    return app



#!/usr/bin/env python
"""
This script opens an output file produced by Abinit (usually in netcdf format but
other files are supported as well). By default the script starts an interactive ipython
session so that one can interact with the file and call its methods.
Alternatively, it is possible to generate automatically a jupyter notebook to execute code.
"""
from __future__ import unicode_literals, division, print_function, absolute_import

import sys
import os
import io
import argparse
import tempfile

from monty.os.path import which
from monty.termcolor import cprint
from monty.functools import prof_main
from abipy import abilab


def make_and_open_notebook(options):
    """
    Generate an jupyter notebook and open it in the browser.
    Return system exit code.

    Raise:
        RuntimeError if jupyther is not in $PATH
    """
    import os
    import nbformat
    nbf = nbformat.v4
    nb = nbf.new_notebook()

    nb.cells.extend([
        nbf.new_markdown_cell("## This is an auto-generated notebook for %s" % os.path.relpath(options.filepath)),
        nbf.new_code_cell("""\
from __future__ import print_function, division, unicode_literals, absolute_import
%matplotlib notebook
import numpy as np
#import seaborn as sns
#sns.set(context='notebook', style='darkgrid', palette='deep',
#        font='sans-serif', font_scale=1, color_codes=False, rc=None)
from abipy import abilab\
"""),

    nbf.new_code_cell("abifile = abilab.abiopen('%s')" % options.filepath)
    ])

    import io, tempfile
    _, nbpath = tempfile.mkstemp(prefix="abinb_", suffix='.ipynb', dir=os.getcwd(), text=True)

    with io.open(nbpath, 'wt', encoding="utf8") as f:
        nbformat.write(nb, f)

    if which("jupyter") is None:
        raise RuntimeError("Cannot find jupyter in PATH. Install it with `pip install`")

    if options.foreground:
        return os.system("jupyter notebook %s" % nbpath)
    else:
        fd, tmpname = tempfile.mkstemp(text=True)
        print(tmpname)
        cmd = "jupyter notebook %s" % nbpath
        print("Executing:", cmd)
        print("stdout and stderr redirected to %s" % tmpname)
        import subprocess
        process = subprocess.Popen(cmd.split(), shell=False, stdout=fd, stderr=fd)
        cprint("pid: %s" % str(process.pid), "yellow")


def get_epilog():
    s = """\
Usage example:

    abiopen.py FILE        => Open file in ipython shell.
    abiopen.py FILE -nb    => Generate jupyter notebook.
    abiopen.py FILE -p     => Print info on object to terminal.
    abiopen.py FILE -e     => Generate matplotlib figures automatically.
                              Use -sns to activate seaborn settings.

`FILE` is any file supported by abipy/pymatgen e.g Netcdf files, Abinit input, POSCAR, xsf ...
Use `-v` to increase verbosity level (can be supplied multiple times e.g -vv).

File extensions supported:
"""
    return s + abilab.abiopen_ext2class_table()

def get_parser(with_epilog=False):
    parser = argparse.ArgumentParser(epilog=get_epilog() if with_epilog else "",
                                     formatter_class=argparse.RawDescriptionHelpFormatter)

    parser.add_argument('--loglevel', default="ERROR", type=str,
        help="Set the loglevel. Possible values: CRITICAL, ERROR (default), WARNING, INFO, DEBUG")
    parser.add_argument('-V', '--version', action='version', version=abilab.__version__)

    parser.add_argument('-v', '--verbose', default=0, action='count', # -vv --> verbose=2
        help='verbose, can be supplied multiple times to increase verbosity')

    parser.add_argument("filepath", help="File to open. See table below for the list of supported extensions.")

    # notebook option
    parser.add_argument('-nb', '--notebook', action='store_true', default=False, help="Open file in jupyter notebook")
    parser.add_argument('--foreground', action='store_true', default=False,
        help="Run jupyter notebook in the foreground.")

    # print option
    parser.add_argument('-p', '--print', action='store_true', default=False, help="Print python object and return.")

    # expose option.
    parser.add_argument('-e', '--expose', action='store_true', default=False,
        help="Open file and generate matplotlib figures automatically by calling expose method.")
    parser.add_argument("-s", "--slide-mode", default=False, action="store_true",
        help="Iterate over figures. Expose all figures at once if not given on the CLI.")
    parser.add_argument("-t", "--slide-timeout", type=int, default=None,
        help="Close figure after slide-timeout seconds (only if slide-mode). Block if not specified.")
    parser.add_argument('-sns', "--seaborn", const="paper", default=None, action='store', nargs='?', type=str,
        help='Use seaborn settings. Accept value defining context in ("paper", "notebook", "talk", "poster"). Default: paper')
    parser.add_argument('-mpl', "--mpl-backend", default=None,
        help=("Set matplotlib interactive backend. "
              "Possible values: GTKAgg, GTK3Agg, GTK, GTKCairo, GTK3Cairo, WXAgg, WX, TkAgg, Qt4Agg, Qt5Agg, macosx."
              "See also: https://matplotlib.org/faq/usage_faq.html#what-is-a-backend."))

    return parser


@prof_main
def main():
    def show_examples_and_exit(err_msg=None, error_code=1):
        """Display the usage of the script."""
        sys.stderr.write(get_epilog())
        if err_msg:
            sys.stderr.write("Fatal Error\n" + err_msg + "\n")
        sys.exit(error_code)

    parser = get_parser(with_epilog=True)

    # Parse the command line.
    try:
        options = parser.parse_args()
    except Exception:
        show_examples_and_exit(error_code=1)

    # loglevel is bound to the string value obtained from the command line argument.
    # Convert to upper case to allow the user to specify --loglevel=DEBUG or --loglevel=debug
    import logging
    numeric_level = getattr(logging, options.loglevel.upper(), None)
    if not isinstance(numeric_level, int):
        raise ValueError('Invalid log level: %s' % options.loglevel)
    logging.basicConfig(level=numeric_level)

    if options.verbose > 2:
        print(options)

    if options.mpl_backend is not None:
        # Set matplotlib backend
        import matplotlib
        matplotlib.use(options.mpl_backend)

    if options.seaborn:
        # Use seaborn settings.
        import seaborn as sns
        sns.set(context=options.seaborn, style='darkgrid', palette='deep',
                font='sans-serif', font_scale=1, color_codes=False, rc=None)

    if not os.path.exists(options.filepath):
        raise RuntimeError("%s: no such file" % options.filepath)

    if not options.notebook:
        abifile = abilab.abiopen(options.filepath)

        if options.print:
            # Print object to terminal.
            if hasattr(abifile, "to_string"):
                print(abifile.to_string(verbose=options.verbose))
            else:
                print(abifile)
            return 0

        elif options.expose:
            # Generate matplotlib plots automatically.
            if hasattr(abifile, "to_string"):
                print(abifile.to_string(verbose=options.verbose))
            else:
                print(abifile)

            if hasattr(abifile, "expose"):

                abifile.expose(slide_mode=options.slide_mode, slide_timeout=options.slide_timeout,
                               verbose=options.verbose)
            else:
                if not hasattr(abifile, "yield_figs"):
                    raise TypeError("Object of type `%s` does not implement (expose or yield_figs methods" % type(abifile))
                from abipy.tools.plotting import MplExpose
                with MplExpose(slide_mode=options.slide_mode, slide_timeout=options.slide_timeout,
                               verbose=options.verbose) as e:
                    e(abifile.yield_figs())

            return 0

        # Start ipython shell with namespace
        # Use embed because I don't know how to show a header with start_ipython.
        import IPython
        IPython.embed(header="The Abinit file is bound to the `abifile` variable.\nTry `print(abifile)`")

    else:
        # Call specialized method if the object is a NotebookWriter
        # else generate simple notebook by calling `make_and_open_notebook`
        cls = abilab.abifile_subclass_from_filename(options.filepath)
        if hasattr(cls, "make_and_open_notebook"):
            if hasattr(cls, "__exit__"):
                with abilab.abiopen(options.filepath) as abifile:
                    return abifile.make_and_open_notebook(foreground=options.foreground)
            else:
                abifile = abilab.abiopen(options.filepath)
                return abifile.make_and_open_notebook(foreground=options.foreground)
        else:
            return make_and_open_notebook(options)

    return 0


if __name__ == "__main__":
    sys.exit(main())



from __future__ import print_function, division, unicode_literals, absolute_import

import os
import wx
import abipy.gui.awx as awx

from collections import OrderedDict
from abipy.flowtk import AbinitTimerSection, AbinitTimerParser


class AbinitTimerFrame(awx.Frame):
    """
    Frame to control to plot the timing data of a *single* ABINIT run.
    """
    def __init__(self, parent, filepath, **kwargs):
        """
        Args:
            parent:
                parent window
            filepath:
                Abinit output file.
        """
        filepath = os.path.abspath(filepath)
        if "title" not in kwargs:
            kwargs["title"] = "Abinit Timer: %s" % os.path.basename(filepath)

        super(AbinitTimerFrame, self).__init__(parent, **kwargs)

        try:
            self.timer = AbinitTimerParser()
            self.timer.parse(filepath)

        except Exception as exc:
            raise awx.Error(str(exc))

        self.BuildUi()

    def BuildUi(self):

        # Set callbacks (bound methods of AbiTimerData).
        self.plot_types = OrderedDict([
            ("pie", self.timer.plot_pie),
            ("stacked_hist", self.timer.plot_stacked_hist),
            #("raw_data", self.OnRawData),
        ])

        keys = AbinitTimerSection.NUMERIC_FIELDS

        main_sizer = wx.BoxSizer(wx.VERTICAL)

        hsizer = wx.BoxSizer(wx.HORIZONTAL)

        plot_label = wx.StaticText(self, -1, "plot type:", wx.DefaultPosition, wx.DefaultSize, 0)
        plot_label.Wrap(-1)
        hsizer.Add(plot_label, 0, wx.ALIGN_CENTER_VERTICAL | wx.TOP | wx.BOTTOM | wx.LEFT, 5)

        self.plot_cbox = wx.ComboBox(self, -1, "pie", wx.DefaultPosition, wx.DefaultSize, self.plot_types.keys(),
                                     0)
        hsizer.Add(self.plot_cbox, 0, wx.ALL, 5)

        key_label = wx.StaticText(self, -1, "key:", wx.DefaultPosition, wx.DefaultSize, 0)
        key_label.Wrap(-1)
        hsizer.Add(key_label, 0, wx.ALIGN_CENTER_VERTICAL | wx.TOP | wx.BOTTOM | wx.LEFT, 5)

        self.key_cbox = wx.ComboBox(self, -1, "wall_time", wx.DefaultPosition, wx.DefaultSize, keys, 0)
        hsizer.Add(self.key_cbox, 0, wx.ALL, 5)

        main_sizer.Add(hsizer, 0, wx.ALIGN_CENTER_HORIZONTAL, 5)

        plot_button = wx.Button(self, -1, "Plot", wx.DefaultPosition, wx.DefaultSize, 0)
        plot_button.Bind(wx.EVT_BUTTON, self.OnClick)
        main_sizer.Add(plot_button, 0, wx.ALL | wx.ALIGN_CENTER_HORIZONTAL, 5)

        self.SetSizerAndFit(main_sizer)

    def OnClick(self, event):
        callback = self.plot_types[self.plot_cbox.GetValue()]
        kwargs = dict(
            key=str(self.key_cbox.GetValue())
        )
        callback(**kwargs)

    #def OnRawData(self, **kwargs):
    #    """Return a string with the timing data."""
    #    table = self.timer.totable()
    #    strio = StringIO.StringIO()
    #    print(table, out=strio)
    #    return strio.getvalue()


class MultiTimerFrame(awx.Frame):
    """
    Frame to control to plot the timing data of a *single* ABINIT run.
    """
    def __init__(self, parent, timers, **kwargs):
        """
        Args:
            parent:
                parent window
            timers:
        """
        super(MultiTimerFrame, self).__init__(parent, **kwargs)
        self.timers = timers

        self.BuildUi()

    def BuildUi(self):
        # Set callbacks (bound methods of AbinitTimer).
        self.plot_types = OrderedDict([
            ("efficiency", self.timers.plot_efficiency),
            ("stacked_hist", self.timers.plot_stacked_hist),
            ("pie", self.timers.plot_pie),
            #("raw_data", self.OnRawData),
        ])

        keys = AbinitTimerSection.NUMERIC_FIELDS

        main_sizer = wx.BoxSizer(wx.VERTICAL)

        hsizer = wx.BoxSizer(wx.HORIZONTAL)

        plot_label = wx.StaticText(self, -1, "plot type:", wx.DefaultPosition, wx.DefaultSize, 0)
        plot_label.Wrap(-1)
        hsizer.Add(plot_label, 0, wx.ALIGN_CENTER_VERTICAL | wx.TOP | wx.BOTTOM | wx.LEFT, 5)

        self.plot_cbox = wx.ComboBox(self, -1, "stacked_hist", wx.DefaultPosition, wx.DefaultSize, self.plot_types.keys(),
                                     0)
        hsizer.Add(self.plot_cbox, 0, wx.ALL, 5)

        key_label = wx.StaticText(self, -1, "key:", wx.DefaultPosition, wx.DefaultSize, 0)
        key_label.Wrap(-1)
        hsizer.Add(key_label, 0, wx.ALIGN_CENTER_VERTICAL | wx.TOP | wx.BOTTOM | wx.LEFT, 5)

        self.key_cbox = wx.ComboBox(self, -1, "wall_time", wx.DefaultPosition, wx.DefaultSize, keys, 0)
        hsizer.Add(self.key_cbox, 0, wx.ALL, 5)

        main_sizer.Add(hsizer, 0, wx.ALIGN_CENTER_HORIZONTAL, 5)

        plot_button = wx.Button(self, -1, "Plot", wx.DefaultPosition, wx.DefaultSize, 0)
        plot_button.Bind(wx.EVT_BUTTON, self.OnClick)
        main_sizer.Add(plot_button, 0, wx.ALL | wx.ALIGN_CENTER_HORIZONTAL, 5)

        self.SetSizerAndFit(main_sizer)

    def OnClick(self, event):
        callback = self.plot_types[self.plot_cbox.GetValue()]
        kwargs = dict(
            key=str(self.key_cbox.GetValue())
        )
        callback(**kwargs)

    #def OnRawData(self, **kwargs):
    #    strings = []
    #    for timer in self.timers:
    #        table = timer.totable()
    #        strio = StringIO.StringIO()
    #        print(table, out=strio)
    #        strings.append(strio.get_value)
    #    return strings




# coding: utf-8
"""
Flow for the computation of Grunesein parameters with Abinit DFPT and finite differences.

WARNING: This code must be tested more carefully.
"""
from __future__ import print_function, division, unicode_literals, absolute_import

from abipy.core.structure import Structure

from .flows import Flow
from .works import Work


class GruneseinenFlow(Flow):
    """
    This work compute the Grüneisen parameters with the DFPT part of Abinit and
    finite differences in anaddb.

    It is necessary to run three phonon calculations with DFPT.
    One is calculated at the equilibrium volume and the remaining two are calculated
    at the slightly larger volume and smaller volume than the equilibrium volume.
    The unitcells at these volumes have to be fully relaxed under the constraint of each volume.
    """
    @classmethod
    def from_gs_input(cls, gsinp, voldelta, workdir=None, manager=None):
        """
        Build the work from an :class:`AbinitInput` object representing a GS calculations.

	Args:
	    gsinp:
		:class:`AbinitInput` object representing a GS calculation in the initial unit cell.
	    voldelta:
                Absolute increment for unit cell volume. The three volumes are:
                     [v0 - voldelta, v0, v0 + voldelta] where v0 is taken from gsinp.structure.

	Return:
        """
        new = cls(workdir=workdir, manager=manager)

	# Save arguments that will be used to call phonopy for creating
        # the supercells with the displacements once the three volumes have been relaxed.
        #new.scdims = np.array(scdims)
        #if new.scdims.shape != (3,):
        #    raise ValueError("Expecting 3 int in scdims but got %s" % str(new.scdims))
        #new.phonopy_kwargs = phonopy_kwargs if phonopy_kwargs is not None else {}
        #new.displ_kwargs = displ_kwargs if displ_kwargs is not None else {}

        # Build three tasks for structural optimization at constant volume.
        v0 = gsinp.structure.volume
        if voldelta <= 0:
            raise ValueError("voldelta must be > 0 but got %s" % voldelta)
        volumes = [v0 - voldelta, v0, v0  + voldelta]
        if any(v <= 0 for v in volumes):
            raise ValueError("volumes must be > 0 but got %s" % str(volumes))

        for vol in volumes:
            # Build new structure
            new_lattice = gsinp.structure.lattice.scale(vol)
            new_structure = Structure(new_lattice, gsinp.structure.species, gsinp.structure.frac_coords)
            new_input = gsinp.new_with_structure(new_structure)
            # Set variables for structural optimization at constant volume.
            new_input.pop_tolerances()
            new_input.set_vars(optcell=3, ionmov=3, tolvrs=1e-10, toldff=1.e-6)
            new_input.set_vars_ifnotin(ecutsm=0.5, dilatmx=1.05)
            new.register_relax_task(new_input)

        return new


class GruneseinenWork(Work):

    def on_all_ok(self):
        """
        This method is called once the `Work` is completed i.e. when all the tasks
        have reached status S_OK.
        """
        self.add_phonopy_works_and_build()
        return super(GruneisenWork, self).on_all_ok()

    def add_phonopy_works_and_build(self):
        """
        Get relaxed structures from the tasks, build Phonopy works with supercells
        constructed from the new structures, add them to the flow and build new directories.
        """
        for i, task in enumerate(self):
            relaxed_structure = task.get_final_structure()
            gsinp = task.input.new_with_structure(relaxed_structure)

            work = PhonopyWork.from_gs_input(gsinp, self.scdims,
                                             phonopy_kwargs=self.phonopy_kwargs,
                                             displ_kwargs=self.displ_kwargs)

            self.flow.register_work(work)
            # Tell the work to copy the results to e.g. `flow/outdir/w0/minus`
            dst = os.path.join(self.pos_str, {0: "minus", 1: "orig", 2: "plus"}[i])
            work.cpdata2dst = self.flow.outdir.path_in(dst)

        # Allocate new works and update the pickle database.
        self.flow.allocate()
        self.flow.build_and_pickle_dump()


#class RelaxAndAddPhGammaWork(RelaxWork):
#    """
#    Work for structural relaxations. The first task relaxes the atomic position
#    while keeping the unit cell parameters fixed. The second task uses the final
#    structure to perform a structural relaxation in which both the atomic positions
#    and the lattice parameters are optimized.
#    """
#
#    def on_all_ok(self):
#        """
#        Here I extend the implementation of super in order to create a new workflow
#        for phonons with the optimized structural parameters.
#        """
#        results = super(RelaxAndAddPhGammaWork, self).on_all_ok()
#
#        # Get the relaxed structure.
#        relax_task = self[1]
#        final_structure = relax_task.get_final_structure()
#
#        # Use new structure in GS + DFPT runs and change some values.
#        scf_input = relax_task.input.deepcopy()
#        scf_input.set_structure(final_structure)
#
#        # Remove input variables that can enter into conflict with DFPT.
#        scf_input.pop_tolerances()
#        scf_input.pop_par_vars()
#        scf_input.pop_irdvars()
#        scf_input.pop_vars(["ionmov", "optcell", "ntime", "dilatmx"])
#        scf_input.set_vars(tolwfr=1e-20, nstep=80, nbdbuf=4)
#        #nval = scf_input.num_valence_electrons
#
#        # Build GS work and Phonon Work
#        work = PhononDojoWork.from_scf_input(scf_input, self.dojo_qpt)
#        for task in work[1:]:
#            task.set_vars(prtwf=-1)
#
#        # Monkey patch work
#        work.dojo_kppa = self.dojo_kppa
#        work.dojo_qpt = self.dojo_qpt
#        work.ecut = self.ecut
#        work.dojo_pawecutdg = self.dojo_pawecutdg
#        work.dojo_include_soc = self.dojo_include_soc
#        work._dojo_trial = "phgamma" if not self.dojo_include_soc else "phgamma_soc"
#        work._pseudo = self.dojo_pseudo
#
#        self.flow.register_work(work)
#        # Allocate new tasks and update the pickle database.
#        self.flow.allocate()
#        self.flow.build_and_pickle_dump()
#
#        return results



from __future__ import print_function, division, absolute_import #, unicode_literals

import string
import warnings
import collections
import numpy as np


__all__ = [
    'InputVariable',
]

_SPECIAL_DATASET_INDICES = (':', '+', '?')

_DATASET_INDICES = ''.join(list(string.digits) + list(_SPECIAL_DATASET_INDICES))

_INTERNAL_DATASET_INDICES = ('__s', '__i', '__a')

_SPECIAL_CONVERSION = zip(_INTERNAL_DATASET_INDICES, _SPECIAL_DATASET_INDICES)

_UNITS = {
    'bohr' : 1.0,
    'angstrom' : 1.8897261328856432,
    'hartree' : 1.0,
    'Ha' : 1.0,
    'eV' : 0.03674932539796232,
}


#def convert_number(value):
#    """
#    Converts some object to a float or a string.
#    If the argument is an integer or a float, returns the same object.
#    If the argument is a string, tries to convert to an integer,
#    then to a float.
#    The string '1.0d-03' will be treated the same as '1.0e-03'
#    """
#    if isinstance(value, (float, int)):
#        return value
#
#    elif isinstance(value, str):
#
#        if is_number(value):
#            try:
#                val = int(value)
#            except ValueError:
#                val = float(value)
#            return val
#
#        else:
#            val = value.replace('d', 'e')
#            if is_number(val):
#                val = float(val)
#                return val
#            else:
#                raise ValueError('convert_number failed')
#
#    else:
#        raise ValueError('convert_number failed')


class InputVariable(object):
    """
    An Abinit input variable.
    """
    def __init__(self, name, value, units='', valperline=3):

        self._name = name
        self.value = value
        self._units = units

        # Maximum number of values per line.
        self.valperline = valperline
        if name in ['bdgw']:
            #TODO Shouldn't do that
            self.valperline = 2

        if (is_iter(self.value) and isinstance(self.value[-1], str) and self.value[-1] in _UNITS):
            self.value = list(self.value)
            self._units = self.value.pop(-1)

    def get_value(self):
        """Return the value."""
        if self.units:
            return list(self.value) + [self.units]
        else:
            return self.value

    @property
    def name(self):
        return self._name

    @property
    def basename(self):
        """Return the name trimmed of any dataset index."""
        basename = self.name
        return basename.rstrip(_DATASET_INDICES)

    @property
    def dataset(self):
        """Return the dataset index in string form."""
        return self.name.split(self.basename)[-1]

    @property
    def units(self):
        """Return the units."""
        return self._units

    def __str__(self):
        """Declaration of the variable in the input file."""
        value = self.value
        if value is None or not str(value):
            return ''

        var = self.name
        line = ' ' + var

        # By default, do not impose a number of decimal points
        floatdecimal = 0

        # For some inputs, impose number of decimal points...
        if any(inp in var for inp in ('xred', 'xcart', 'rprim', 'qpt', 'kpt')):
            #TODO Shouldn't do that
            floatdecimal = 16

        # ...but not for those
        if any(inp in var for inp in ('ngkpt', 'kptrlatt', 'ngqpt', 'ng2qpt')):
            #TODO Shouldn't do that
            floatdecimal = 0

        if isinstance(value, np.ndarray):
            n = 1
            for i in np.shape(value):
                n *= i
            value = np.reshape(value, n)
            value = list(value)

        # values in lists
        if isinstance(value, (list, tuple)):

            # Reshape a list of lists into a single list
            if all(isinstance(v, (list, tuple)) for v in value):
                line += self.format_list2d(value, floatdecimal)

            else:
                # Maximum number of values per line.
                #valperline = 3
                #if any(inp in var for inp in ['bdgw']):
                #    #TODO Shouldn't do that
                #    valperline = 2

                line += self.format_list(value, floatdecimal)

        # scalar values
        else:
            line += ' ' + str(value)

        # Add units
        if self.units:
            line += ' ' + self.units

        return line

    def format_scalar(self, val, floatdecimal=0):
        """
        Format a single numerical value into a string
        with the appropriate number of decimal.
        """
        sval = str(val)
        if sval.lstrip('-').lstrip('+').isdigit() and floatdecimal == 0:
            return sval

        try:
            fval = float(val)
        except:
            return sval

        if fval == 0 or (abs(fval) > 1e-3 and abs(fval) < 1e4):
            form = 'f'
            addlen = 5
        else:
            form = 'e'
            addlen = 8

        ndec = max(len(str(fval-int(fval)))-2, floatdecimal)
        ndec = min(ndec, 10)

        sval = '{v:>{l}.{p}{f}}'.format(v=fval, l=ndec+addlen, p=ndec, f=form)

        sval = sval.replace('e', 'd')

        return sval

    def format_list2d(self, values, floatdecimal=0):
        """Format a list of lists."""
        lvals = flatten(values)

        # Determine the representation
        if all(isinstance(v, int) for v in lvals):
            type_all = int
        else:
            try:
                for v in lvals:
                    float(v)
                type_all = float
            except:
                type_all = str

        # Determine the format
        width = max(len(str(s)) for s in lvals)
        if type_all == int:
            formatspec = '>{0}d'.format(width)
        elif type_all == str:
            formatspec = '>{0}'.format(width)
        else:

            # Number of decimal
            maxdec = max(len(str(f-int(f)))-2 for f in lvals)
            ndec = min(max(maxdec, floatdecimal), 10)

            if all(f == 0 or (abs(f) > 1e-3 and abs(f) < 1e4) for f in lvals):
                formatspec = '>{w}.{p}f'.format(w=ndec+5, p=ndec)
            else:
                formatspec = '>{w}.{p}e'.format(w=ndec+8, p=ndec)

        line = '\n'
        for L in values:
            for val in L:
                line += ' {v:{f}}'.format(v=val, f=formatspec)
            line += '\n'

        return line.rstrip('\n')

    def format_list(self, values, floatdecimal=0):
        """
        Format a list of values into a string.
        The result might be spread among several lines.
        """
        line = ''

        # Format the line declaring the value
        for i, val in enumerate(values):
            line += ' ' + self.format_scalar(val, floatdecimal)
            if self.valperline is not None and (i+1) % self.valperline == 0:
                line += '\n'

        # Add a carriage return in case of several lines
        if '\n' in line.rstrip('\n'):
            line = '\n' + line

        return line.rstrip('\n')

    #@staticmethod
    #def string_to_value(sval):
    #    """
    #    Interpret a string variable and attempt to return a value of the
    #    appropriate type.  If all else fails, return the initial string.
    #    """
    #    value = None

    #    try:
    #        for part in sval.split():

    #            if '*' in part:
    #                # cases like istwfk *1
    #                if part[0] == '*':
    #                    value = None
    #                    break

    #                # cases like acell 3*3.52
    #                else:
    #                    n = int(part.split('*')[0])
    #                    f = convert_number(part.split('*')[1])
    #                    if value is None:
    #                        value = []
    #                    value += n * [f]
    #                    continue

    #            # Fractions
    #            if '/' in part:
    #                (num, den) = (float(part.split('/')[i]) for i in range(2))
    #                part = num / den

    #            # Unit
    #            if part in _UNITS.keys():

    #                if value is None:
    #                    warnings.warn("Could not apply the unit token '%s'." % part)
    #                elif isinstance(value, list):
    #                    value.append(part)
    #                else:
    #                    value = [value, part]

    #                # Convert
    #                if False:
    #                    if isinstance(value, list):
    #                        for i in range(len(value)):
    #                            value[i] *= _UNITS[part]
    #                    elif isinstance(value, str):
    #                        value = None
    #                        break
    #                    else:
    #                        value *= _UNITS[part]

    #                continue

    #            # Convert
    #            try:
    #                val = convert_number(part)
    #            except:
    #                val = part

    #            if value is None:
    #                value = val
    #            elif isinstance(value, list):
    #                value.append(val)
    #            else:
    #                value = [value, val]
    #    except:
    #        value = None

    #    if value is None:
    #        value = sval

    #    return value

    #@classmethod
    #def from_str(cls, bigstring):
    #    """Return an instance from a string declaration."""
    #    parts = bigstring.split()

    #    # Perform checks on the string
    #    if len(parts) < 2 or (parts[-1] in _UNITS and len(parts) < 3):
    #        msg = '\n'.join(['Unable to initialize variable from string:',
    #                         bigstring, 'not enough tokens.'])
    #        raise ValueError(msg)
    #    elif not parts[0].isalpha():
    #        msg = '\n'.join(['Unable to initialize variable from string:',
    #                         bigstring, 'no valid variable name found.'])
    #        raise ValueError(msg)

    #    # Make the name
    #    name = parts.pop(0)
    #    #name = cls.declared_to_internal(name)

    #    # Make the units
    #    if parts[-1] in _UNITS:
    #        units = parts.pop(-1)
    #    else:
    #        units = None

    #    value = cls.string_to_value(' '.join(parts))

    #    return cls(name, value, units)


#def is_number(s):
#    """Returns True if the argument can be made a float."""
#    try:
#        float(s)
#        return True
#    except:
#        return False


def is_iter(obj):
    """Return True if the argument is list-like."""
    return hasattr(obj, '__iter__')


def flatten(iterable):
    """Make an iterable flat, i.e. a 1d iterable object."""
    iterator = iter(iterable)
    array, stack = collections.deque(), collections.deque()
    while True:
        try:
            value = next(iterator)
        except StopIteration:
            if not stack:
                return tuple(array)
            iterator = stack.pop()
        else:
            if not isinstance(value, str) \
               and isinstance(value, collections.Iterable):
                stack.append(iterator)
                iterator = iter(value)
            else:
                array.append(value)



#!/usr/bin/env python
import os
from abipy.gui.wxapps import wxapp_showfiles

# Show all *.py files located within the current directory
app = wxapp_showfiles(filenames=None, dirpath=os.path.dirname(__file__), walk=True, wildcard="*.py")
app.MainLoop()



from __future__ import print_function, division, unicode_literals, absolute_import

from pymatgen.io.abinit.tasks import *


class BoxcuttedPhononTask(PhononTask):
    """
    This task compute phonons with a two-step algorithm.
    The first DFPT run is done with low-accuracy settings for boxcutmin and ecut
    The second DFPT run uses boxcutmin 2.0 and normal ecut and restarts from
    the 1WFK file generated previously.
    """
    @classmethod
    def patch_flow(cls, flow):
        for task in flow.iflat_tasks():
            if isinstance(task, PhononTask): task.__class__ = cls

    def setup(self):
        super(BoxcuttedPhononTask, self).setup()
        self.final_dfp_done = False if not hasattr(self, "final_dfp_done") else self.final_dfp_done
        if not self.final_dfp_done:
            # First run: use boxcutmin 1.5 and low-accuracy hints (assume pseudos with hints).
            pseudos = self.input.pseudos
            ecut = max(p.hint_for_accuracy("low").ecut for p in pseudos)
            pawecutdg = max(p.hint_for_accuracy("low").pawecutdg for p in pseudos) if self.input.ispaw else None
            self.set_vars(boxcutmin=1.5, ecut=ecut, pawecutdg=pawecutdg, prtwf=1)

    def _on_ok(self):
        results = super(BoxcuttedPhononTask, self)._on_ok()
        if not self.final_dfp_done:
            # Second run: use exact box and normal-accuracy hints (assume pseudos with hints).
            pseudos = self.input.pseudos
            ecut = max(p.hint_for_accuracy("normal").ecut for p in pseudos)
            pawecutdg = max(p.hint_for_accuracy("normal").pawecutdg for p in pseudos) if self.input.ispaw else None
            self.set_vars(boxcutmin=2.0, ecut=ecut, pawecutdg=pawecutdg, prtwf=-1)
            self.finalized = True
            self.final_dfp_done = True
            self.restart()

        return results




#!/usr/bin/env python
from __future__ import print_function, division, unicode_literals, absolute_import

import numpy as np

from abipy.core.kpoints import issamek

def slow_mesh2ibz(structure, bz, ibz):
    #print("bz",bz)
    #print("ibz",ibz)
    bz2ibz = -np.ones(len(bz), dtype=np.int)

    for ik_bz, kbz in enumerate(bz):
        #print("kbz",kbz)
        found = False

        for ik_ibz, kibz in enumerate(ibz):
            #print("kibz",kibz)
            if found: break
            for symmop in structure.spacegroup:
                krot = symmop.rotate_k(kibz.frac_coords)
                if issamek(krot, kbz):
                    bz2ibz[ik_bz] = ik_ibz
                    found = True
                    break

    return bz2ibz


def map_mesh2ibz(structure, mpdivs, shifts, ibz):
    """
    This function computes the mapping between the
    points in the full BZ and the points in the IBZ.

    Args:
        structure:
            pymatgen `Structure` instance.
        mpdivs
            The three MP divisions
        shifts:
            Array-like object with the MP shift.
        ibz:
            array-like object with the reduced coordinates of the
            points in the IBZ.

    Returns
        ndarray array that maps the full mesh onto ibz i.e.

            bzmap[idx_bz] = id_ibz

        where idx_bz is the index the BZ and idx_ibz the index in the IBZ.
    """
    # Build bz grid.
    mpdivs = np.asarray(mpdivs)

    shifts = np.reshape(shifts, (-1,3))
    num_shifts = len(shifts)

    from collections import namedtuple
    class KSymmetryTables(namedtuple("KSymmetryTables", "bz2ibz ktabi ktabo")):
        pass

    spg = structure.spacegroup

    timrev = 2 if spg.has_timerev else 1
    spg_fdata = spg.to_fortran_arrays()

    # Build mapping.
    ibz_fdata = ibz.to_fortran_arrays()

    tables = num_shifts * [None]

    for ish, shift in enumerate(shifts):
        kbz, count = np.empty((mpdivs.prod(), 3)), 0

        for i in range(mpdivs[0]):
            x = (i + shift[0]) / mpdivs[0]
            for j in range(mpdivs[1]):
                y = (j + shift[1]) / mpdivs[1]
                for k in range(mpdivs[2]):
                    z = (k + shift[2]) / mpdivs[2]
                    kbz[count] = (x, y, z)
                    count += 1

        bz2ibz, ktabi, ktabo, ierr = mod.map_bz2ibz(kibz=ibz_fdata.frac_coords,
                                                    kbz=np.asfortranarray(kbz.T),
                                                    timrev=timrev,
                                                    symrec=spg_fdata.symrec,
                                                    symafm=spg_fdata.symafm
                                                    )
        if ierr != 0:
            msg = "map_bz2ibz returned ierr %d, Kmesh is not symmetric" % ierr
            #raise KmeshNotSymmetricError(msg)
            raise ValueError(msg)

        #for (ik_bz, full_kpt) in enumerate(kbz):
        #    ik_ibz = bz2ibz[ik_bz]
        #    isym = ktabo[ik_bz]
        #    itime = ktabi[ik_bz]
        #    op = spg[isym + itime
        #    krot = op.rotate_k(ibz[ik_ibz].frac_coords, wrap_tows=True)
        #    assert full_kpt == krot

        tables[ish] = KSymmetryTables(bz2ibz=bz2ibz, ktabi=ktabi, ktabo=ktabo)

    return tuple(tables)


def main():
    #import m_kpoints
    #from m_kpoints import m_kpoints  as mod
    #print(m_kpoints.__doc__)
    #v1 = np.zeros(3)
    #v2 = v1 + 1.0
    #print(v1, v2)

    #ans, g0 = mod.isamek(v1, v2)
    #print(ans,g0)

    from abipy.core.kpoints import IrredZone, kmesh_from_mpdivs

    filename = "/Users/gmatteo/Coding/abipy/abipy/data/runs/data_si_ebands/outdata/si_scf_GSR.nc"
    structure = Structure.from_file(filename)

    ibz = IrredZone.from_file(filename)
    #print(ibz)

    mpdivs = [8, 8, 8]
    shifts = [0.0, 0.0, 0.0]

    #tables = map_mesh2ibz(structure, mpdivs, shifts, ibz)

    bz = kmesh_from_mpdivs(mpdivs, shifts)

    ktab = slow_map_mesh2ibz(structure, bz, ibz.frac_coords)

if __name__ == "__main__":
    main()



# coding: utf-8
"""
Objects common to the other eph modules.
"""
from __future__ import print_function, division, unicode_literals, absolute_import

import numpy as np

from collections import OrderedDict
from monty.functools import lazy_property
from abipy.electrons.ebands import ElectronsReader


class BaseEphReader(ElectronsReader):
    """
    Provides methods common to the netcdf files produced by the EPH code.
    See Abinit docs for the meaning of the variables.
    """

    @lazy_property
    def ddb_ngqpt(self):
        """Q-Mesh for DDB file."""
        return self.read_value("ddb_ngqpt")

    @lazy_property
    def ngqpt(self):
        """Effective Q-mesh used in to compute integrals (ph_linewidts, e-ph self-energy)."""
        return self.read_value("ngqpt")

    @lazy_property
    def ph_ngqpt(self):
        """Q-mesh for Phonon DOS, interpolated A2F ..."""
        return self.read_value("ph_ngqpt")

    @lazy_property
    def eph_ngqpt_fine(self):
        """Q-mesh for interpolated DFPT potentials"""
        return self.read_value("eph_ngqpt_fine")

    @lazy_property
    def common_eph_params(self):
        """
        Read basic parameters (scalars) from the netcdf files produced by the EPH code and cache them
        """
        od = OrderedDict([
            ("ddb_nqbz", np.prod(self.ddb_ngqpt)),
            ("eph_nqbz_fine", np.prod(self.eph_ngqpt_fine)),
            ("ph_nqbz", np.prod(self.ph_ngqpt)),
        ])

        for vname in ["eph_intmeth", "eph_fsewin", "eph_fsmear", "eph_extrael", "eph_fermie"]:
            value = self.read_value(vname)
            if vname in ("eph_intmeth",):
                value = int(value)
            else:
                value = float(value)
            od[vname] = value

        return od



from __future__ import print_function, division, unicode_literals, absolute_import

import wx
import collections
import fnmatch
import abipy.gui.awx as awx

from monty.string import list_strings
from abipy.electrons import ElectronBandsPlotter, ElectronDosPlotter, MdfPlotter, SigresPlotter
from abipy.gui.electronswx import ElectronDosDialog


class FileCheckBoxPanel(awx.Panel):
    """A panel with a list of filepaths and checkboxes."""
    def __init__(self, parent, filepaths, **kwargs):
        """
        Args:
            parent:
                Parent window.
            filepaths:
                String or List of strings with filepaths.
        """
        super(FileCheckBoxPanel, self).__init__(parent, -1, **kwargs)

        self.all_filepaths = list_strings(filepaths)

        self.BuildUi()

    def BuildUi(self):
        main_sizer = wx.BoxSizer(wx.VERTICAL)

        static_sizer = wx.StaticBoxSizer(wx.StaticBox(self, -1, "Files"), wx.VERTICAL)

        self.check_boxes = collections.OrderedDict()

        self.wildcards = ["*"]

        for path in self.all_filepaths:
            if not self._SelectFilename(path): continue
            assert path not in self.check_boxes
            cbox = wx.CheckBox(self, -1, path, wx.DefaultPosition, wx.DefaultSize, 0)
            cbox.SetValue(True)
            static_sizer.Add(cbox, 0, wx.ALL | wx.EXPAND, 5)
            self.check_boxes[path] = cbox

        main_sizer.Add(static_sizer, 1, wx.EXPAND, 5)

        # Add buttons to (select|deselect) all checkboxes.
        hsizer = wx.BoxSizer(wx.HORIZONTAL)

        all_button = wx.Button(self, -1, "Select all", wx.DefaultPosition, wx.DefaultSize, 0)
        all_button.Bind(wx.EVT_BUTTON, self.OnSelectAll)
        hsizer.Add(all_button, 0, wx.ALL, 5)

        deselect_button = wx.Button(self, -1, "Deselect all", wx.DefaultPosition, wx.DefaultSize, 0)
        deselect_button.Bind(wx.EVT_BUTTON, self.OnDeselectAll)
        hsizer.Add(deselect_button, 0, wx.ALL, 5)

        filter_label = wx.StaticText(self, -1, "Filter:", wx.DefaultPosition, wx.DefaultSize, 0)
        filter_label.Wrap(-1)
        hsizer.Add(filter_label, 0, wx.ALIGN_CENTER_VERTICAL | wx.TOP | wx.BOTTOM | wx.LEFT, 5)

        wildcard_choices = ["*", "*.nc"]
        self.filter_combobox = wx.ComboBox(self, wx.ID_ANY, "*", wx.DefaultPosition, wx.DefaultSize, wildcard_choices,
                                           0)
        self.filter_combobox.Bind(wx.EVT_COMBOBOX, self.OnFilterComboBox)
        self.filter_combobox.Bind(wx.EVT_TEXT_ENTER, self.OnFilterComboBox)
        hsizer.Add(self.filter_combobox, 0, wx.ALL, 5)

        main_sizer.Add(hsizer, 0, wx.ALIGN_CENTER_HORIZONTAL, 5)

        self.SetSizerAndFit(main_sizer)

    def _SelectFilename(self, filename):
        for wcard in self.wildcards:
            if not fnmatch.fnmatch(filename, wcard):
                return False
        return True

    def OnFilterComboBox(self, event):
        self.wildcards = self.filter_combobox.GetValue().split("|")

        for (filepath, cbox) in self.check_boxes.items():
            if self._SelectFilename(filepath):
                cbox.SetValue(True)
            else:
                cbox.SetValue(False)

    def OnSelectAll(self, event):
        for cbox in self.check_boxes.values():
            cbox.SetValue(True)

    def OnDeselectAll(self, event):
        for cbox in self.check_boxes.values():
            cbox.SetValue(False)

    def GetSelectedFilepaths(self):
        """
        Return the list of filepaths selected by the user.
        Main entry point for client code.
        """
        selected = []

        for path, cbox in self.check_boxes.items():
            if cbox.GetValue():
                selected.append(path)

        return selected


class ComparisonFrame(awx.Frame):
    """
    This frame allows the user to select/deselect a list of files and to produce plots
    for all the files selected. Useful for convergence studies.
    """

    def __init__(self, parent, dirpaths=None, filepaths=None, wildcard=None, **kwargs):
        """
        Args:
            parent:
                parent window
            dirpaths
                List of directory names
            filepaths
                List of filepaths.
            wildcard
                Regular expression for selecting files.
        """
        super(ComparisonFrame, self).__init__(parent, -1, **kwargs)

        # TODO
        #self.dirpaths = dirpaths if dirpaths is not None else []
        #self.filepaths = filepaths if filepaths is not None else []
        #
        #self.dirpaths = map(os.path.abspath, self.dirpaths)
        #self.filepaths = map(os.path.abspath, self.filepaths)
        #
        #self.wildcard = wildcard if wildcard is not None else ""

        main_sizer = wx.BoxSizer(wx.VERTICAL)

        hsizer = wx.BoxSizer(wx.HORIZONTAL)

        st1 = wx.StaticText(self, -1, "Quantity:", wx.DefaultPosition, wx.DefaultSize, 0)
        st1.Wrap(-1)
        hsizer.Add(st1, 0, wx.ALIGN_CENTER_VERTICAL | wx.TOP | wx.BOTTOM | wx.LEFT, 5)

        plotter_choices = ["ebands", "edos", "mdf", "sigres"]
        self.plotter_cbox = wx.ComboBox(self, -1, "ebands", wx.DefaultPosition, wx.DefaultSize, plotter_choices, 0)
        hsizer.Add(self.plotter_cbox, 0, wx.ALL, 5)

        compare_button = wx.Button(self, -1, "Compare", wx.DefaultPosition, wx.DefaultSize, 0)
        compare_button.Bind(wx.EVT_BUTTON, self.OnCompareButton)
        hsizer.Add(compare_button, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)

        main_sizer.Add(hsizer, 0, wx.ALIGN_CENTER_HORIZONTAL, 5)

        self.panel = FileCheckBoxPanel(self, filepaths)
        main_sizer.Add(self.panel, 1, wx.EXPAND, 5)

        self.SetSizerAndFit(main_sizer)

    def OnCompareButton(self, event):
        selected_files = self.panel.GetSelectedFilepaths()

        choice = self.plotter_cbox.GetValue()

        try:
            if choice == "ebands":
                plotter = ElectronBandsPlotter()

                for filepath in selected_files:
                    plotter.add_ebands_from_file(filepath)

                plotter.plot()

            elif choice == "edos":
                # Open dialog to ask the user the DOS parameters.
                dos_dialog = ElectronDosDialog(None)

                if dos_dialog.ShowModal() == wx.ID_OK:
                    p = dos_dialog.GetParams()

                    plotter = ElectronDosPlotter()

                    for filepath in selected_files:
                        plotter.add_edos_from_file(filepath, **p)

                    plotter.plot()

                dos_dialog.Destroy()

            elif choice == "mdf":
                plotter = MdfPlotter()

                for filepath in selected_files:
                    plotter.add_mdf_from_file(filepath, mdf_type="exc")

                plotter.plot()

            elif choice == "sigres":
                plotter = SigresPlotter()

                plotter.add_files(selected_files)

                plotter.plot_qpgaps()
                #plotter.plot_qpenes()

            else:
                awx.showErrorMessage(self, message="No function registered for choice %s" % choice)

        except Exception:
            awx.showErrorMessage(self)


def wxapp_comparison(dirpaths=None, filepaths=None, wildcard=None):
    app = awx.App()
    frame = ComparisonFrame(None, dirpaths=dirpaths, filepaths=filepaths, wildcard=wildcard)
    frame.Show()
    return app



#!/usr/bin/env python
import os
import abipy
import abipy.data

import abipy.gui.wxapps as wxapps

datadir = abipy.data.dirpath
filepaths = [os.path.join(datadir, f) for f in os.listdir(datadir)]

app = wxapps.wxapp_comparison(filepaths=filepaths).MainLoop()



"""Widgets for browsing and/or analyzing the output files produced by Abinit."""
from __future__ import print_function, division, unicode_literals, absolute_import

import os
import wx
import abipy.gui.awx as awx
import wx.lib.mixins.listctrl as listmix

from collections import namedtuple
from monty.string import list_strings, is_string
from monty.fnmatch import WildCard
from abipy.gui.popupmenus import popupmenu_for_filename


def frameclass_from_filepath(filepath):
    """
    Factory function that returns the class of the viewer (wx frame) associated to the file.
    Returns None if no viewer has been registered for this filepath.
    """
    from abipy.gui.wfkviewer import WfkViewerFrame
    from abipy.gui.sigresviewer import SigresViewerFrame
    from abipy.gui.gsrviewer import GsrViewerFrame
    from abipy.gui.mdfviewer import MdfViewerFrame
    from abipy.gui.editor import MyEditorFrame
    from abipy.gui.wxncview import NcViewerFrame

    VIEWER_FRAMES = {
        "WFK-etsf.nc": WfkViewerFrame,
        "SIGRES.nc": SigresViewerFrame,
        "GSR.nc": GsrViewerFrame,
        "MDF.nc": MdfViewerFrame,
        ".abi": MyEditorFrame,
        ".abo": MyEditorFrame,
        ".log": MyEditorFrame,
        ".sh": MyEditorFrame,
        ".err": MyEditorFrame,
        ".files": MyEditorFrame,
    }

    ext = filepath.split("_")[-1]
    try:
        return VIEWER_FRAMES[ext]

    except KeyError:
        root, ext = os.path.splitext(filepath)
        try:
            return VIEWER_FRAMES[ext]
        except KeyError:
            # No frame registered for the file: return NcViewerFrame if netcdf file else None
            return NcViewerFrame if filepath.endswith(".nc") else None


def frame_from_filepath(parent, filepath):
    """
    Factory function that returns the viewer (wx frame) associated to the file.
    None if no viewer has been registered for this filename.
    """
    frame_class = frameclass_from_filepath(filepath)
    return frame_class if frame_class is None else frame_class(parent, filepath)


class NcFileDirCtrl(wx.GenericDirCtrl):
    def __init__(self, *args, **kwargs):

        if "filter" not in kwargs:
            kwargs["filter"] = "Netcdf files (*.nc)|*.nc|All files (*.*)|*.*"

        if "dir" not in kwargs:
            kwargs["dir"] = os.getcwd()

        if "style" not in kwargs:
            kwargs["style"] = wx.TR_MULTIPLE

        super(NcFileDirCtrl, self).__init__(*args, **kwargs)

        self.Bind(wx.EVT_TREE_ITEM_ACTIVATED, self.OnItemActivated)
        self.Bind(wx.EVT_TREE_ITEM_RIGHT_CLICK, self.OnRightClick)

    def OnItemActivated(self, event):
        path = self.GetFilePath()
        if not path: return

        frame = frame_from_filepath(self, path)
        if frame is not None:
            frame.Show()

    def OnRightClick(self, event):
        path = self.GetFilePath()
        if not path: return

        # Open the popup menum then destroy it to avoid mem leak.
        popmenu = popupmenu_for_filename(self, path)
        self.PopupMenu(popmenu, event.GetPoint())
        popmenu.Destroy()


class MyListCtrl(wx.ListCtrl, listmix.ListCtrlAutoWidthMixin):
    """ Mixin class to resize the last column appropriately."""
    def __init__(self, parent, **kwargs):
        wx.ListCtrl.__init__(self, parent, id=-1, style=wx.LC_REPORT | wx.BORDER_SUNKEN, **kwargs)
        listmix.ListCtrlAutoWidthMixin.__init__(self)


class FileDataObj(namedtuple("FileDataObj", "filename type directory")):
    """The fields of the row in `FileListPanel`"""
    @property
    def abspath(self):
        """Absolute path ofthe file."""
        return os.path.join(self.directory, self.filename)

    #@property
    #def basename(self)
    #    """Basename of the file."""
    #    return os.path.basename(self.abspath))

    @property
    def relpath(self):
        """Relative path"""
        return os.path.relpath(self.abspath)

    @property
    def reldirpath(self):
        """Relative path of the directory"""
        return os.path.relpath(self.directory)

    @classmethod
    def from_abspath(cls, abspath):
        return cls(filename=os.path.basename(abspath),
                   type=os.path.splitext(abspath)[-1],
                   directory=os.path.dirname(abspath))


class FileListPanel(awx.Panel, listmix.ColumnSorterMixin):
    """
    This panel shows a list of files (strings), supports column sorting
    and provides specific popup menus for the different type of files
    (file type detection is based on file extensions).
    """
    def __init__(self, parent, filepaths, **kwargs):
        """
        Args:
            parent:
                parent window
            filepaths:
                List of file paths.
        """
        super(FileListPanel, self).__init__(parent, -1, **kwargs)

        if filepaths is not None and is_string(filepaths):
            filepaths = [filepaths]

        self.filepaths = filepaths if filepaths is not None else []
        self.filepaths = map(os.path.abspath, self.filepaths)

        self.BuildUi()

    def BuildUi(self):
        self.file_list = file_list = MyListCtrl(self)

        file_list.Bind(wx.EVT_LIST_ITEM_RIGHT_CLICK, self.OnRightClick)
        file_list.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.OnItemActivated)

        self.columns = columns = ["filename", "type", "reldirpath"]

        # Used to store the Max width in pixels for the data in the column.
        column_widths = [awx.get_width_height(self, s)[0] for s in columns]

        self.id2filedata = {}
        for (index, colname) in enumerate(columns):
            file_list.InsertColumn(index, colname)

        for filepath in self.filepaths:
            entry = self.AppendFilepath(filepath)

            w = [awx.get_width_height(self, s)[0] for s in entry]
            column_widths = map(max, zip(w, column_widths))

        for (index, colname) in enumerate(columns):
            #file_list.SetColumnWidth(index, wx.LIST_AUTOSIZE)
            file_list.SetColumnWidth(index, column_widths[index])

        # Now that the list exists we can init the other base class, see wx/lib/mixins/listctrl.py
        self.itemDataMap = self.id2filedata
        listmix.ColumnSorterMixin.__init__(self, len(columns))
        self.Bind(wx.EVT_LIST_COL_CLICK, self.OnColClick, self.file_list)

        # Pack
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(file_list, 1, wx.ALL | wx.EXPAND, 5)

        self.SetSizerAndFit(sizer)

    def HasAbsPath(self, abspath):
        """True if abspath is already present."""
        return abspath in [f.abspath for f in self.id2filedata.values()]

    def GetListCtrl(self):
        """Used by the ColumnSorterMixin, see wx/lib/mixins/listctrl.py"""
        return self.file_list

    def AppendFilepath(self, abspath):
        """Add a file to the panel."""
        if self.HasAbsPath(abspath):
            awx.showErrorMessage(self, message="%s is already in the list" % abspath)

        return self._AppendFilepath(abspath)

    def _AppendFilepath(self, abspath):
        filedata = FileDataObj.from_abspath(abspath)

        # We use next as entry id because we want to be able
        # to sort the columns with the mixin ColumnSorterMixin.
        next = len(self.id2filedata)
        entry_id = next
        entry = [getattr(filedata, attr) for attr in self.columns]
        self.file_list.Append(entry)
        self.file_list.SetItemData(next, entry_id)
        self.id2filedata[entry_id] = filedata

        return entry

    def OnItemActivated(self, event):
        currentItem = event.m_itemIndex
        fd = self.id2filedata[self.file_list.GetItemData(currentItem)]

        frame = frame_from_filepath(self, fd.abspath)
        if frame is not None:
            frame.Show()

    def OnRightClick(self, event):
        currentItem = event.m_itemIndex
        if currentItem == -1:
            return

        fd = self.id2filedata[self.file_list.GetItemData(currentItem)]
        # Open the popup menu then destroy it to avoid mem leak.
        menu = popupmenu_for_filename(self, fd.abspath)

        if menu is not None:
            self.PopupMenu(menu, event.GetPoint())
            menu.Destroy()

    def OnColClick(self, event):
        event.Skip()


class FileListFrame(awx.Frame):
    def __init__(self, parent, dirpaths=None, filepaths=None, walk=True, wildcard="", **kwargs):
        """
        Args:
            parent:
                parent window
            dirpaths:
                List of directories to scan.
            filepaths
                List of filepaths (absolute paths).
            walk:
                True if we have to browse all files and directories starting from filepaths.
            wildcard
                Regular expressions for selecting files (tokens are separated by |).
        """
        super(FileListFrame, self).__init__(parent, -1, **kwargs)

        if dirpaths is not None:
            dirpaths = map(os.path.abspath, list_strings(dirpaths))
        else:
            dirpaths = []

        if filepaths is not None:
            filepaths = map(os.path.abspath, list_strings(filepaths))
        else:
            filepaths = []

        wildcard = WildCard(wildcard)

        self.all_filepaths = filepaths

        if walk:
            for dirpath in dirpaths:
                for root, dirnames, filenames in os.walk(dirpath):
                    fnames = [os.path.join(root, f) for f in filenames]
                    self.all_filepaths += wildcard.filter(fnames)
        else:
            # Select only the files in dirpaths.
            for dirpath in dirpaths:
                fnames = [os.path.join(dirpat, f) for f in os.listdir(dirpath)]
                fnames = filter(os.path.isfile, fnames)
                self.all_filepaths += wildcard.filter(fnames)

        self.BuildUi()

    def BuildUi(self):
        self.main_sizer = main_sizer = wx.BoxSizer(wx.VERTICAL)

        panel = FileListPanel(self, filepaths=self.all_filepaths)
        main_sizer.Add(panel, 1, wx.EXPAND, 5)

        hsizer = wx.BoxSizer(wx.HORIZONTAL)

        filter_label = wx.StaticText(self, -1, "Filter:", wx.DefaultPosition, wx.DefaultSize, 0)
        filter_label.Wrap(-1)
        hsizer.Add(filter_label, 0, wx.ALIGN_CENTER_VERTICAL | wx.TOP | wx.BOTTOM | wx.LEFT, 5)

        # Combobox to enter shell patterns.
        wildcard_choices = ["*", "*.nc", "*.abo", "*.log"]
        self.filter_combobox = wx.ComboBox(self, id=-1, value="*", style=wx.TE_PROCESS_ENTER, choices=wildcard_choices)
        self.filter_combobox.SetToolTipString("Shell patterns separated by |")

        self.filter_combobox.Bind(wx.EVT_COMBOBOX, self.OnFilterComboBox)
        self.filter_combobox.Bind(wx.EVT_TEXT_ENTER, self.OnFilterComboBox)

        hsizer.Add(self.filter_combobox, 0, wx.ALL, 5)

        main_sizer.Add(hsizer, 0, wx.ALIGN_CENTER_HORIZONTAL, 5)

        self.SetSizer(main_sizer)
        self.Layout()

    def OnFilterComboBox(self, event):
        wildcard = WildCard(self.filter_combobox.GetValue())

        select_files = wildcard.filter(self.all_filepaths)
        panel = FileListPanel(self, filepaths=select_files)

        main_sizer = self.main_sizer
        main_sizer.Hide(0)
        main_sizer.Remove(0)
        main_sizer.Insert(0, panel, 1, wx.EXPAND, 5)

        self.Layout()
        #self.Fit()


# FIXME: Rewrite these classes:
class DirBrowserFrame(awx.Frame):
    def __init__(self, parent, **kwargs):
        super(DirBrowserFrame, self).__init__(parent, -1)
        NcFileDirCtrl(self, -1, **kwargs)


def wxapp_dirbrowser(dirpath):
    """Standalone application.for DirBrowser."""
    if dirpath is None:
        dirpath = " "
    else:
        dirpath = os.path.abspath(dirpath)

    app = awx.App()
    frame = DirBrowserFrame(None, dir=dirpath)
    frame.Show()
    return app


def wxapp_listbrowser(dirpaths=None, filepaths=None, wildcard=""):
    """Standalone application ListBroweser."""
    app = awx.App()
    frame = FileListFrame(None, dirpaths=dirpaths, filepaths=filepaths, wildcard=wildcard)
    frame.Show()
    return app



#!/usr/bin/env python
r"""
G0W0 convergence study
======================

G0W0 convergence study wrt ecuteps and the number of bands in W.
"""
from __future__ import print_function, division, unicode_literals, absolute_import

import sys
import os
import numpy as np

import abipy.abilab as abilab
import abipy.data as abidata
import abipy.flowtk as flowtk


def make_inputs(paral_kgb=1):
    """
    Returns a tuple of 4 input files for SCF, NSCF, SCR, SIGMA calculations.
    These files are then used as templates for the convergence study
    wrt ecuteps and the number of bands in W.
    """
    multi = abilab.MultiDataset(abidata.structure_from_ucell("SiC"),
                                pseudos=abidata.pseudos("14si.pspnc", "6c.pspnc"), ndtset=4)

    ecut = 12
    global_vars = dict(
        ecut=ecut,
        istwfk="*1",
        paral_kgb=paral_kgb,
        gwpara=2,
        iomode=1,
    )

    ecuteps = 4
    ngkpt = [4, 4, 4]
    shiftk = [0, 0, 0]

    multi.set_vars(global_vars)
    multi.set_kmesh(ngkpt=ngkpt, shiftk=shiftk)

    # SCF
    multi[0].set_vars(
        nband=10,
        tolvrs=1.e-8,
    )

    # NSCF
    multi[1].set_vars(
        nband=25,
        tolwfr=1.e-8,
        iscf=-2
    )

    # SCR
    multi[2].set_vars(
        optdriver=3,
        ecutwfn=ecut,
        nband=20,
        symchi=1,
        inclvkb=0,
        ecuteps=ecuteps,
    )

    # SIGMA
    multi[3].set_vars(
        optdriver=4,
        nband=20,
        ecutwfn=ecut,
        ecutsigx=ecut,
        #ecutsigx=(4*ecut), ! This is problematic
        symsigma=1,
        ecuteps=ecuteps,
        )

    multi[3].set_kptgw(kptgw=[[0,0,0], [0.5, 0, 0]], bdgw=[1, 8])

    return multi.split_datasets()


def build_flow(options):
    # Working directory (default is the name of the script with '.py' removed and "run_" replaced by "flow_")
    if not options.workdir:
        options.workdir = os.path.basename(__file__).replace(".py", "").replace("run_", "flow_")

    # Get our templates
    scf_inp, nscf_inp, scr_inp, sig_inp = make_inputs()

    ecuteps_list = np.arange(2, 8, 2)
    max_ecuteps = max(ecuteps_list)

    flow = flowtk.Flow(workdir=options.workdir, manager=options.manager)

    # Band structure work to produce the WFK file
    bands = flowtk.BandStructureWork(scf_inp, nscf_inp)
    flow.register_work(bands)

    # Build a work made of two SCR runs with different value of nband
    # Use max_ecuteps for the dielectric matrix (sigma tasks will
    # read a submatrix when we test the convergence wrt to ecuteps.
    scr_work = flowtk.Work()

    for inp in scr_inp.generate(nband=[10, 15]):
        inp.set_vars(ecuteps=max_ecuteps)
        scr_work.register_scr_task(inp, deps={bands.nscf_task: "WFK"})

    flow.register_work(scr_work)

    # Do a convergence study wrt ecuteps, each work is connected to a
    # different SCR file computed with a different value of nband.

    # Build a list of sigma inputs with different ecuteps
    sigma_inputs = list(sig_inp.generate(ecuteps=ecuteps_list))

    for scr_task in scr_work:
        sigma_conv = flowtk.SigmaConvWork(wfk_node=bands.nscf_task, scr_node=scr_task, sigma_inputs=sigma_inputs)
        flow.register_work(sigma_conv)

    return flow

# This block generates the thumbnails in the Abipy gallery.
# You can safely REMOVE this part if you are using this script for production runs.
if os.getenv("READTHEDOCS", False):
    __name__ = None
    import tempfile
    options = flowtk.build_flow_main_parser().parse_args(["-w", tempfile.mkdtemp()])
    #build_flow(options).plot_networkx(with_edge_labels=True, tight_layout=True)
    build_flow(options).graphviz_imshow()


@flowtk.flow_main
def main(options):
    """
    This is our main function that will be invoked by the script.
    flow_main is a decorator implementing the command line interface.
    Command line args are stored in `options`.
    """
    return build_flow(options)

if __name__=="__main__":
    sys.exit(main())



#!/usr/bin/env python
"""Benchmark for phonon calculation with DFPT.""" 
from __future__ import division, print_function, unicode_literals, absolute_import

import sys
import numpy as np
import abipy.abilab as abilab
import abipy.flowtk as flowtk
import abipy.data as abidata

from itertools import product
from abipy.benchmarks import bench_main, BenchmarkFlow


def make_inputs(paw=False):
    pseudos = abidata.pseudos("56ba.psp_mod", "22ti.psp_mod", "8o.psp_mod")
    #pseudos = abidata.pseudos("56ba.psp_mod", "22ti.psp_mod", "8o.pspnc")
    if paw: raise NotImplementedError("PAW")

    # SLAB ending TiO2 double layer. paralelectric configuration
    # N=9
    # Supercell and atoms

    xcart= np.fromstring("""
0.0000000000E+00  0.0000000000E+00 -4.2633349730E+00
3.7794522658E+00  3.7794522658E+00 -3.2803418097E+00
0.0000000000E+00  3.7794522658E+00 -3.6627278067E+00
3.7794522658E+00  0.0000000000E+00  6.5250113947E-01
0.0000000000E+00  3.7794522658E+00 -1.0555036964E-01
3.7794522658E+00  3.7794522658E+00  3.2682166278E-01
0.0000000000E+00  0.0000000000E+00  3.9815918094E+00
3.7794522658E+00  3.7794522658E+00  4.0167907030E+00
3.7794522658E+00  0.0000000000E+00  7.7541444349E+00
0.0000000000E+00  3.7794522658E+00  7.6664087705E+00
3.7794522658E+00  3.7794522658E+00  7.7182324796E+00
0.0000000000E+00  0.0000000000E+00  1.1412913350E+01
3.7794522658E+00  3.7794522658E+00  1.1416615533E+01
3.7794522658E+00  0.0000000000E+00  1.5117809063E+01
0.0000000000E+00  3.7794522658E+00  1.5117809063E+01
3.7794522658E+00  3.7794522658E+00  1.5117809063E+01
0.0000000000E+00  0.0000000000E+00  1.8822704777E+01
3.7794522658E+00  3.7794522658E+00  1.8819002593E+01
3.7794522658E+00  0.0000000000E+00  2.2481473692E+01
0.0000000000E+00  3.7794522658E+00  2.2569209355E+01
3.7794522658E+00  3.7794522658E+00  2.2517385647E+01
0.0000000000E+00  0.0000000000E+00  2.6254026317E+01
3.7794522658E+00  3.7794522658E+00  2.6218827423E+01
3.7794522658E+00  0.0000000000E+00  2.9583116986E+01
0.0000000000E+00  3.7794522658E+00  3.0341168496E+01
3.7794522658E+00  3.7794522658E+00  2.9908796464E+01
0.0000000000E+00  0.0000000000E+00  3.4498953099E+01
3.7794522658E+00  3.7794522658E+00  3.3515959935E+01
0.0000000000E+00  3.7794522658E+00  3.3898345933E+01
""", sep=" ").reshape((-1,3))

    # Crystal structure.
    structure = abilab.Structure.from_abivars(
        #acell="4.0 4.0 28.0 Angstrom",
        acell=abilab.ArrayWithUnit([4.0, 4.0, 28], "ang").to("bohr"),
        rprim=np.eye(3), 
        typat=[int(i) for i in "3 3 2 3 3 2 1 3 3 3 2 1 3 3 3 2 1 3 3 3 2 1 3 3 3 2 3 3 2".split()],
        znucl=[56, 22, 8],
        xcart=xcart,
    )

    multi = abilab.MultiDataset(structure, pseudos, ndtset=2)

    # Global variables used both for the GS and the DFPT run.
    multi.set_vars(
        #electronic structure
        nband=120,
        ecut=15.0,
        pawecutdg=30.0 if paw else None,
        nstep=80,
        ngkpt=[4,4,1],
        shiftk=[0,0,0],
        paral_kgb=1,
        timopt=-1,
        prtden=0,
    )

    gs_inp, ph_inp = multi.split_datasets()

    gs_inp.set_vars(tolvrs=1e-6, kptopt=1)

    ph_inp.set_vars(
        kptopt=3,
        rfphon=1,
        irdwfk=1,
        rfatpol=[1, 1],
        rfdir=[0, 0, 1],
        nqpt=1,
        qpt=[0.0, 0.25, 0.0],
        prtwf=0,
        #tolwfr=1.0e-22,
        toldfe=1.0e-9,
        tolrde=0.0, # This is a development input variable, used in the present case to avoid load unbalance
                    # when studying the scaling with respect to the number of cores. Do not define it in
                    # your production runs
    )

    return gs_inp, ph_inp


def build_flow(options):
    gs_inp, ph_inp = make_inputs()

    flow = BenchmarkFlow(workdir=options.get_workdir(__file__), remove=options.remove)
    gs_work = flowtk.Work()
    gs_work.register_scf_task(gs_inp)
    flow.register_work(gs_work)
    flow.exclude_from_benchmark(gs_work)

    # Get the list of possible parallel configurations from abinit autoparal.
    max_ncpus, min_eff = options.max_ncpus, options.min_eff
    print("Getting all autoparal confs up to max_ncpus:", max_ncpus," with efficiency >=", min_eff)

    pconfs = ph_inp.abiget_autoparal_pconfs(max_ncpus, autoparal=1)
    if options.verbose: print(pconfs)

    work = flowtk.Work()
    for conf, omp_threads in product(pconfs, options.omp_list):
        if not options.accept_conf(conf, omp_threads): continue

        manager = options.manager.new_with_fixed_mpi_omp(conf.mpi_procs, omp_threads)
        inp = ph_inp.new_with_vars(conf.vars)
        work.register_phonon_task(inp, manager=manager, deps={gs_work[0]: "WFK"})

    print("Found %d configurations" % len(work))
    flow.register_work(work)

    return flow.allocate()


@bench_main
def main(options):
    if options.info:
        # print doc string and exit.
        print(__doc__)
        return 
    return build_flow(options)


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
r"""
Flow for E-PH calculations
==========================

This flow computes the phonon linewidths and the Eliashberg function in Al.
"""
from __future__ import print_function, division, unicode_literals, absolute_import

import sys
import os
import abipy.data as abidata
import abipy.abilab as abilab
import abipy.flowtk as flowtk


def build_flow(options):
    # Working directory (default is the name of the script with '.py' removed and "run_" replaced by "flow_")
    if not options.workdir:
        options.workdir = os.path.basename(__file__).replace(".py", "").replace("run_", "flow_")

    # Preparatory run for E-PH calculations.
    # The sequence of datasets makes the ground states and
    # all of the independent perturbations of the single Al atom
    # for the irreducible qpoints in a 4x4x4 grid.
    # Note that the q-point grid must be a sub-grid of the k-point grid (here 8x8x8)
    pseudos = abidata.pseudos("Al.oncvpsp")

    structure = abilab.Structure.from_abivars(
        acell=3 * [7.5],
        rprim=[0.0, 0.5, 0.5,
               0.5, 0.0, 0.5,
               0.5, 0.5, 0.0],
        typat=1,
        xred=[0.0, 0.0, 0.0],
        ntypat=1,
        znucl=13,
    )

    same_structure = abilab.Structure.fcc(a=7.5, species=["Al"], units="bohr")
    assert same_structure == structure

    gs_inp = abilab.AbinitInput(structure, pseudos)

    gs_inp.set_vars(
        istwfk="*1",
        ecut=8.0,
        nband=5,
        occopt=7,    # include metallic occupation function with a small smearing
        tsmear=0.04,
        tolvrs=1e-7,
        #timeopt=-1,
    )

    # The kpoint grid is minimalistic to keep the calculation manageable.
    gs_inp.set_kmesh(
        ngkpt=[8, 8, 8],
        shiftk=[0.0, 0.0, 0.0],
        #kptopt=3,
    )

    # Phonon calculation with 4x4x4
    ddb_ngqpt = [4, 4, 4]
    qpoints = gs_inp.abiget_ibz(ngkpt=ddb_ngqpt, shiftk=[0, 0, 0], kptopt=1).points

    flow = flowtk.Flow(options.workdir, manager=options.manager)
    work0 = flow.register_scf_task(gs_inp)

    ph_work = flowtk.PhononWork.from_scf_task(work0[0], qpoints)
    flow.register_work(ph_work)

    eph_work = flow.register_work(flowtk.Work())
    eph_deps = {work0[0]: "WFK", ph_work: ["DDB", "DVDB"]}

    for eph_ngqpt_fine in [(4, 4, 4,), (8, 8, 8)]:
        # Build input file for E-PH run.
        eph_inp = gs_inp.new_with_vars(
            optdriver=7,
            ddb_ngqpt=ddb_ngqpt,       # q-mesh used to produce the DDB file (must be consistent with DDB data)
            eph_intmeth=2,             # Tetra method
            eph_fsewin="0.8 eV",       # Energy window around Ef
            eph_mustar=0.12,           # mustar parameter
            eph_ngqpt_fine=eph_ngqpt_fine, # Interpolate DFPT potentials if != ddb_ngqpt
        )

        # Set q-path for phonons and phonon linewidths.
        eph_inp.set_qpath(20)

        # TODO: Define wstep and smear
        # Set q-mesh for phonons DOS and a2F(w)
        eph_inp.set_phdos_qmesh(nqsmall=16, method="tetra")
        eph_work.register_eph_task(eph_inp, deps=eph_deps)

    flow.allocate(use_smartio=True)

    return flow


# This block generates the thumbnails in the Abipy gallery.
# You can safely REMOVE this part if you are using this script for production runs.
if os.getenv("GENERATE_SPHINX_GALLERY", False):
    __name__ = None
    import tempfile
    options = flowtk.build_flow_main_parser().parse_args(["-w", tempfile.mkdtemp()])
    build_flow(options).plot_networkx(tight_layout=True)


@flowtk.flow_main
def main(options):
    """
    This is our main function that will be invoked by the script.
    flow_main is a decorator implementing the command line interface.
    Command line args are stored in `options`.
    """
    return build_flow(options)


if __name__ == "__main__":
    sys.exit(main())



# coding: utf-8
"""Widgets for Pandas Dataframes."""
from __future__ import print_function, division, unicode_literals, absolute_import

import pandas as pd
import ipywidgets as ipw
import abipy.display.utils as ut

from functools import wraps


@wraps(pd.DataFrame.plot)
def plot(data, **kwargs):

    def plot_dataframe(x, y, kind, sharex, sharey, subplots, grid, legend,
                      logx, logy, loglog, colorbar, sort_columns): # pragma: no cover

        x, y = ut.widget2py(x, y)
        sharex, colorbar = ut.str2bool_or_none(sharex, colorbar)
        data.plot(x=x, y=y, kind=kind, subplots=subplots, sharex=None, sharey=sharey,
                layout=None, figsize=None, use_index=True, title=None, grid=grid, legend=legend, style=None,
                logx=logx, logy=logy, loglog=loglog, xticks=None, yticks=None, xlim=None, ylim=None,
                rot=None, fontsize=None, colormap=colorbar, table=False, yerr=None, xerr=None, secondary_y=False,
                sort_columns=sort_columns, **kwargs)
                # There's a typo in the documentation (colorbar/colormap!)
        #import matplotlib.pyplot as plt
        #return plt.gcf()

    allcols = ["None"] + list(data.keys())
    return ipw.interact_manual(
                plot_dataframe,
                x=allcols,
                y=allcols,
                sharex=["None", "True", "False"],
                sharey=False,
                kind=["line", "bar", "barh", "hist", "box", "kde", "density", "area", "pie", "scatter", "hexbin"],
                subplots=False,
                grid=True,
                legend=True,
                logx=False,
                logy=False,
                loglog=False,
                colorbar=["None", "True", "False"],
                sort_columns=False,
            )


#!/usr/bin/env python
r"""
Band structure Flow with factory functions
==========================================

This example show how to build a flow to compute the band structure and the DOS of silicon.
Input files are automatically generated with factory functions designed for automatic calculations.
"""
from __future__ import print_function, division, unicode_literals, absolute_import

import sys
import os
import abipy.data as abidata
import abipy.flowtk as flowtk

from abipy import abilab

def build_flow(options):
    # Working directory (default is the name of the script with '.py' removed and "run_" replaced by "flow_")
    if not options.workdir:
        options.workdir = os.path.basename(__file__).replace(".py", "").replace("run_", "flow_")

    # Initialize structure and pseudos.
    structure = abilab.Structure.from_file(abidata.cif_file("si.cif"))
    pseudos = abidata.pseudos("14si.pspnc")

    # Initialize the flow.
    flow = flowtk.Flow(workdir=options.workdir, manager=options.manager)

    # Use the ebands_input factory function to build a MultiDataset.
    # keyword args are optional (default values are given or computed automatically, see docs).
    multi = abilab.ebands_input(structure, pseudos, kppa=40, dos_kppa=80,
                                nscf_nband=6, ndivsm=10, ecut=6, spin_mode="unpolarized")

    work = flowtk.BandStructureWork(scf_input=multi[0], nscf_input=multi[1], dos_inputs=multi[2])
    flow.register_work(work)

    return flow


# This block generates the thumbnails in the Abipy gallery.
# You can safely REMOVE this part if you are using this script for production runs.
if os.getenv("READTHEDOCS", False):
    __name__ = None
    import tempfile
    options = flowtk.build_flow_main_parser().parse_args(["-w", tempfile.mkdtemp()])
    #build_flow(options).plot_networkx(with_edge_labels=True, tight_layout=True)
    build_flow(options).graphviz_imshow()


@flowtk.flow_main
def main(options):
    """
    This is our main function that will be invoked by the script.
    flow_main is a decorator implementing the command line interface.
    Command line args are stored in `options`.
    """
    return build_flow(options)


if __name__ == "__main__":
    sys.exit(main())

############################################################################
#
# Run the script with:
#
#     run_hs_si_ebands.py -s
#
# Use:
#
#    abirun.py flow_h_si_ebands deps
#
# To print the connections among the tasks.
#
# .. code-block:: bash
#
#     <ScfTask, node_id=241095, workdir=flow_ht_si_ebands/w0/t0>
#
#     <NscfTask, node_id=241096, workdir=flow_ht_si_ebands/w0/t1>
#       +--<ScfTask, node_id=241095, workdir=flow_ht_si_ebands/w0/t0>
#
#     <NscfTask, node_id=241097, workdir=flow_ht_si_ebands/w0/t2>
#       +--<ScfTask, node_id=241095, workdir=flow_ht_si_ebands/w0/t0>
#
# ``w0/t1`` is a band structure calculation along a k-path
#  while ``w0/t2`` produced KS eigenvalues in the IBZ for the DOS.
#
#  You can use ipython to plot the electronic bands with the DOS:
#
# .. code-block:: ipython
#
#    In [1]: %matplotlib
#    In [2]: from abipy import abilab
#    In [3]: gsr_kpath = abilab.abiopen("flow_ht_si_ebands/w0/t1/outdata/out_GSR.nc")
#    In [4]: gsr_kmesh = abilab.abiopen("flow_ht_si_ebands/w0/t2/outdata/out_GSR.nc")
#    In [5]: gsr_kpath.ebands.plot_with_edos(gsr_kmesh.ebands.get_edos())
#
# .. image:: https://github.com/abinit/abipy_assets/blob/master/run_ht_si_ebands.png?raw=true
#    :alt: Band structure of Si in the IBZ and along a k-path
#



from __future__ import print_function, division, unicode_literals, absolute_import

import wx
import abipy.gui.awx as awx

from monty.string import is_string
from abipy.core.structure import Structure
from abipy.gui.editor import SimpleTextViewer


class StructureConverterFrame(wx.Frame):
    """
    This frame allows the user to convert the structure to different formats (CIF, POSCAR, ...).
    """
    def __init__(self, parent, obj, **kwargs):
        """
        Args:
            parent:
                Parent window.
            obj:
                Structure object, filename or object with a structure attribute.
        """
        super(StructureConverterFrame, self).__init__(parent, -1, **kwargs)
        self._get_structure(obj)

        panel = wx.Panel(self, id=-1)

        main_sizer = wx.BoxSizer(wx.VERTICAL)

        hsizer = wx.BoxSizer(wx.HORIZONTAL)

        label = wx.StaticText(panel, -1, "Convert to:")
        label.Wrap(-1)

        # list of supported formats.
        formats = ["cif", "POSCAR", "cssr", "json"]
        self.format_choice = wx.Choice(panel, -1, choices=formats)
        self.format_choice.SetSelection(0)

        show_button = wx.Button(panel, -1, "Show", wx.DefaultPosition, wx.DefaultSize, 0)
        show_button.Bind(wx.EVT_BUTTON, self.OnShow)

        save_button = wx.Button(panel, -1, "Save", wx.DefaultPosition, wx.DefaultSize, 0)
        save_button.Bind(wx.EVT_BUTTON, self.OnSave)

        hsizer.Add(label, 0, wx.ALL, 5)
        hsizer.Add(self.format_choice, 0, wx.ALL | wx.ALIGN_CENTER_HORIZONTAL, 5)
        hsizer.Add(show_button, 0, wx.ALL | wx.ALIGN_CENTER_HORIZONTAL, 5)
        hsizer.Add(save_button, 0, wx.ALL | wx.ALIGN_CENTER_HORIZONTAL, 5)

        main_sizer.Add(hsizer, 0, wx.ALL | wx.ALIGN_CENTER_HORIZONTAL, 5)

        str_structure = wx.TextCtrl(panel, -1, str(self.structure), style=wx.TE_MULTILINE|wx.TE_LEFT|wx.TE_READONLY)
        main_sizer.Add(str_structure, 1, wx.ALL | wx.EXPAND, 5)

        panel.SetSizerAndFit(main_sizer)

    def _get_structure(self, obj):
        """Extract the structure from the input object."""
        return Structure.as_structure(obj)

    @property
    def format(self):
        """The format specified by the user."""
        return self.format_choice.GetStringSelection()

    def _convert(self):
        """Returns string with the structure converted in the user-specified format."""
        return self.structure.convert(fmt=self.format)

    def OnShow(self, event):
        s = self._convert()
        SimpleTextViewer(self, text=s, title=self.structure.formula).Show()

    def OnSave(self, event):
        save_dialog = wx.FileDialog(self, "Save %s file" % self.format, "", "",
                                    wildcard="%{format} files (*.{format})|*.{format}".format(format=self.format),
                                    style=wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT)

        if save_dialog.ShowModal() == wx.ID_CANCEL: return

        with open(save_dialog.GetPath(), "w") as fh:
            fh.write(self._convert())


def wxapp_structure_converter(obj):
    """
    Standalong WX application for structure conversion.

    Args:
        obj:
            Structure object, filename or object with a structure attribute.
    """
    app = awx.App()
    frame = StructureConverterFrame(None, obj)
    app.SetTopWindow(frame)
    frame.Show()

    return app



#!/usr/bin/env python
"""
This script visualizes results with external graphical applications.
or convert data from Abinit files (usually netcdf) to other formats.
"""
from __future__ import unicode_literals, division, print_function, absolute_import

import sys
import os
import argparse
import numpy as np

from monty.functools import prof_main
from monty.termcolor import cprint
from abipy import abilab
from abipy.iotools.visualizer import Visualizer
from abipy.tools.plotting import MplExpose, GenericDataFilePlotter


def handle_overwrite(path, options):
    """Exit 1 if file ``path`` exists and not options.force else return path."""
    name_parts = os.path.splitext(path)
    print("Writing %s file:" % name_parts[-1].replace("." , "").upper())
    if os.path.exists(path) and not options.force:
        cprint("Cannot overwrite pre-existent file. Use `-f` options.", "red")
        sys.exit(1)
    return path


def abiview_structure(options):
    """
    Visualize the structure with the specified visualizer. Requires external app
    or optional python modules (mayavi, vtk)
    """
    structure = abilab.Structure.from_file(options.filepath)
    print(structure.to_string(verbose=options.verbose))
    print("Visualizing structure with:", options.appname)
    structure.visualize(appname=options.appname)
    return 0


def abiview_hist(options):
    """
    Visualize structural relaxation/molecular-dynamics run
    from data stored in the HIST.nc file. Requires mayavi.
    """
    with abilab.abiopen(options.filepath) as hist:
        print(hist.to_string(verbose=options.verbose))
        if options.appname is not None:
            hist.visualize(appname=options.appname, to_unit_cell=options.to_unit_cell)
        elif options.xdatcar:
            xpath = options.filepath + ".XDATCAR"
            handle_overwrite(xpath, options)
            hist.write_xdatcar(filepath=xpath, groupby_type=True, overwrite=True,
                               to_unit_cell=options.to_unit_cell)
        else:
            hist.plot()

        return 0


def abiview_data(options):
    """
    Extract data from a generic text file with results
    in tabular format and plot data with matplotlib.
    Multiple datasets are supported.
    No attempt is made to handle metadata (e.g. column name)
    """
    plotter = GenericDataFilePlotter(options.filepath)
    print(plotter.to_string(verbose=options.verbose))
    plotter.plot(use_index=options.use_index)

    return 0


#def abiview_abo(options):
#    """
#    Plot self-consistent iterations extracted from Abinit output file
#    as well as timer data (if present)
#    """
#    with abilab.abiopen(options.filepath) as abo:
#        print(abo.to_string(verbose=options.verbose))
#        abo.plot()
#        #timer = abo.get_timer()
#    return 0


def abiview_dirviz(options):
    """
    Visualize directory tree with graphviz.
    """
    from pymatgen.io.abinit.utils import Dirviz
    import tempfile
    graph = Dirviz(options.filepath).get_cluster_graph(engine=options.engine)
    directory = tempfile.mkdtemp()
    print("Producing source files in:", directory)
    graph.view(directory=directory)

    return 0


def abiview_ebands(options):
    """
    Plot electronic bands if file contains high-symmetry k-path or DOS if k-sampling.
    Accept any file with ElectronBands e.g. GSR.nc, WFK.nc, ...
    """
    with abilab.abiopen(options.filepath) as abifile:
        if options.xmgrace:
            outpath = options.filepath + ".agr"
            abifile.ebands.to_xmgrace(handle_overwrite(outpath, options))
        elif options.bxsf:
            outpath = options.filepath + ".bxsf"
            abifile.ebands.to_bxsf(handle_overwrite(outpath, options))
        else:
            print(abifile.to_string(verbose=options.verbose))
            abifile.expose_ebands(slide_mode=options.slide_mode, slide_timeout=options.slide_timeout,
                                  verbose=options.verbose)

        return 0


def abiview_skw(options):
    """
    Interpolate energies in k-space along a k-path with star-function methods
    Note that the interpolation will likely fail if there are symmetrical k-points in the input set of k-points
    so it's recommended to call this method with energies obtained in the IBZ.
    Accept any file with ElectronBands e.g. GSR.nc, WFK.nc, ...
    """
    ebands = abilab.ElectronBands.as_ebands(options.filepath)
    if not ebands.kpoints.is_ibz:
        cprint("SKW interpolator should be called with energies in the IBZ", "yellow")
    r = ebands.interpolate(lpratio=options.lpratio, line_density=options.line_density, verbose=options.verbose)
    r.ebands_kpath.plot()
    return 0


def abiview_fs(options):
    """
    Extract eigenvaleus in the IBZ from file and visualize Fermi surface with --appname
    """
    with abilab.abiopen(options.filepath) as abifile:
        eb3d = abifile.ebands.get_ebands3d()

        if options.appname == "mpl":
            eb3d.plot_isosurfaces()
        elif options.appname == "xsf":
            eb3d.xcrysden_view()
        elif options.appname == "mayavi":
            eb3d.mvplot_isosurfaces()
        else:
            raise ValueError("Unsupported value for --appname: %s" % str(options.appname))

        return 0


def abiview_ddb(options):
    """
    Invoke Anaddb to compute phonon bands and DOS from the DDB, plot the results.
    """
    with abilab.abiopen(options.filepath) as ddb:
        print(ddb.to_string(verbose=options.verbose))

        # Don't need PHDOS if phononwebsite
        nqsmall = 0 if options.phononwebsite else 10
        ndivsm = 20; asr = 2; chneut = 1; dipdip = 1; dos_method = "tetra"; lo_to_splitting = "automatic"
        print("""
Computing phonon bands and DOS from DDB file with
nqsmall = {nqsmall}, ndivsm = {ndivsm};
asr = {asr}, chneut = {chneut}, dipdip = {dipdip}, lo_to_splitting = {lo_to_splitting}, dos_method = {dos_method}
""".format(**locals()))

        print("Invoking anaddb ...  ", end="")
        phbst_file, phdos_file = ddb.anaget_phbst_and_phdos_files(
            nqsmall=nqsmall, ndivsm=ndivsm, asr=asr, chneut=chneut, dipdip=dipdip, dos_method=dos_method,
            lo_to_splitting=lo_to_splitting, verbose=options.verbose, mpi_procs=1)
        print("Calculation completed.\nResults available in", os.path.dirname(phbst_file.filepath))

        phbands = phbst_file.phbands
        phdos = phdos_file.phdos

        if options.xmgrace:
            outpath = options.filepath + ".agr"
            phbands.to_xmgrace(handle_overwrite(outpath, options))
            return 0
        #elif options.bxsf:
        #    outpath = options.filepath + ".bxsf"
        #    phbands.to_bxsf(handle_overwrite(outpath, options))
        #    return 0
        elif options.phononwebsite:
            return phbands.view_phononwebsite(browser=options.browser, verbose=options.verbose)
        else:
            units = "mev"
            with MplExpose(slide_mode=options.slide_mode, slide_timeout=options.slide_timeout) as e:
                #e(phbst_file.expose())
                #e(phdos_file.expose())
                e(phbands.qpoints.plot(show=False))
                e(phbands.plot_with_phdos(phdos, units=units, show=False))
                e(phbands.plot_colored_matched(units=units, show=False))
                e(phbands.plot_fatbands(units=units, phdos_file=phdos_file, show=False))
                e(phdos.plot(units=units, show=False))
                e(phdos_file.plot_pjdos_type(units=units, show=False))

        phbst_file.close()
        phdos_file.close()

    return 0


def abiview_phbands(options):
    """Plot phonon bands. Accept any file with PhononBands e.g. PHBST.nc, ..."""
    with abilab.abiopen(options.filepath) as abifile:
        if options.xmgrace:
            outpath = options.filepath + ".agr"
            abifile.phbands.to_xmgrace(handle_overwrite(outpath, options))
        #elif options.bxsf:
        #    outpath = options.filepath + ".bxsf"
        #    abifile.phbands.to_bxsf(handle_overwrite(outpath, options))
        #    return 0
        elif options.phononwebsite:
            return abifile.phbands.view_phononwebsite(browser=options.browser)
        else:
            print(abifile.to_string(verbose=options.verbose))
            abifile.expose_phbands(slide_mode=options.slide_mode, slide_timeout=options.slide_timeout,
                           verbose=options.verbose, units="mev")

        return 0


def abiview_denpot(options):
    """
    Plot DEN/POT files
    """
    with abilab.abiopen(options.filepath) as abifile:
        print(abifile.to_string(verbose=options.verbose))
        field = abifile.field
        if options.chgcar:
            if not field.is_density_like:
                cprint("Warning: expecting DENSITY file but received %s" % field.__class__.__name__, "yellow")
            chg_path = options.filepath + ".CHGCAR"
            field.to_chgcar(filename=handle_overwrite(chg_path, options))
        #elif options.cube:
        #    outpath = options.filepath + ".cube"
        #    field.export_to_cube(filename=handle_overwrite(outpath, options), spin="total")
        else:
            abifile.field.visualize(appname=options.appname)
    return 0


def abiview_lobster(options):
    """Analyze lobster output files in directory."""
    from abipy.electrons.lobster import LobsterAnalyzer
    lobana = LobsterAnalyzer.from_dir(os.path.dirname(options.filepath), prefix=options.prefix)
    print(lobana.to_string(verbose=options.verbose))

    if options.ipython:
        # Start ipython shell with namespace
        # Use embed because I don't know how to show a header with start_ipython.
        import IPython
        IPython.embed(header="The LobsterAnalyzer is bound to the `lobana` variable.\nTry `print(lobana)`")

    elif options.notebook:
        return lobana.make_and_open_notebook(foreground=options.foreground)

    else:
        lobana.plot()
        #lobana.plot_coxp_with_dos(from_site_index=[0, 1])

    return 0


def get_epilog():
    return """\
Usage example:

###########
# Structure
###########

    abiview.py structure FILE                ==> Visualize structure with Vesta (default)
    abiview.py structure FILE -a xcrysden    ==> Visualize structure with Xcrysden (default)
    abiview.py hist out_HIST.nc              ==> Plot relaxation/molecular dynamics results with matplotlib.
    abiview.py hist out_HIST.nc -a ovito     ==> Visualize relaxation/molecular dynamics results with ovito.
    abiview.py hist out_HIST.nc --xdatcar    ==> Convert HIST.nc into XDATCAR format (caveat: assume fixed unit cell!)

############
# Text files
############

    abiview.py data FILE     ==> Parse text FILE with data in tabular format and plot arrays.

###########
# Electrons
###########

    abiview.py ebands out_WFK.nc              ==>  Plot electrons bands (or DOS) with matplotlib.
    abiview.py ebands out_GSR.nc --xmgrace    ==>  Generate xmgrace file with electron bands.
    abiview.py fs FILE_WITH_KMESH.nc          ==>  Visualize Fermi surface from netcdf file with electron energies
                                                   on a k-mesh. Use -a xsf to change application e.g. Xcrysden.
    abiview.py skw out_GSR.nc                 ==> Interpolate IBZ energies with star-functions and plot
                                                  interpolated bands.

#########
# Phonons
#########

    abiview.py ddb out_DDB                ==>  Compute ph-bands and DOS from DDB, plot results.
    abiview.py phbands out_PHBST.nc -web  ==>  Visualize phonon bands and displacements with phononwebsite.

###############
# Miscelleanous
###############

  abiview.py dirviz DIRECTORY            ==> Visualize directory tree with graphviz.

  abiview.py lobster DIRECTORY           ==> Visualize Lobster results.

Use `abiview.py --help` for help and `abiview.py COMMAND --help` to get the documentation for `COMMAND`.
Use `-v` to increase verbosity level (can be supplied multiple times e.g -vv).
"""

# TODO
#abiview.py denpot out_DEN.nc              ==>  Visualize density with Vesta.
#abiview.py denpot out_DEN.nc --chgcar     ==>  Convert DEN file into CHGCAR fileformat.

def get_parser(with_epilog=False):

    # Parent parser for common options.
    copts_parser = argparse.ArgumentParser(add_help=False)
    copts_parser.add_argument('filepath', type=str, help="File to visualize.")

    copts_parser.add_argument('--loglevel', default="ERROR", type=str,
        help="Set the loglevel. Possible values: CRITICAL, ERROR (default), WARNING, INFO, DEBUG")
    copts_parser.add_argument('-v', '--verbose', default=0, action='count', # -vv --> verbose=2
        help='verbose, can be supplied multiple times to increase verbosity.')
    copts_parser.add_argument('-sns', "--seaborn", const="paper", default=None, action='store', nargs='?', type=str,
        help='Use seaborn settings. Accept value defining context in ("paper", "notebook", "talk", "poster"). Default: paper')
    copts_parser.add_argument('-mpl', "--mpl-backend", default=None,
        help=("Set matplotlib interactive backend. "
              "Possible values: GTKAgg, GTK3Agg, GTK, GTKCairo, GTK3Cairo, WXAgg, WX, TkAgg, Qt4Agg, Qt5Agg, macosx."
              "See also: https://matplotlib.org/faq/usage_faq.html#what-is-a-backend."))

    # Parent parser for commands supporting MplExpose.
    slide_parser = argparse.ArgumentParser(add_help=False)
    slide_parser.add_argument("-s", "--slide-mode", default=False, action="store_true",
            help="Iterate over figures. Expose all figures at once if not given on the CLI.")
    slide_parser.add_argument("-t", "--slide-timeout", type=int, default=None,
            help="Close figure after slide-timeout seconds (only if slide-mode). Block if not specified.")

    # Parent parser for commands supporting ipython
    ipy_parser = argparse.ArgumentParser(add_help=False)
    ipy_parser.add_argument('-ipy', '--ipython', default=False, action="store_true", help='Invoke ipython terminal.')

    # Parent parser for commands supporting (jupyter notebooks)
    nb_parser = argparse.ArgumentParser(add_help=False)
    nb_parser.add_argument('-nb', '--notebook', default=False, action="store_true", help='Generate jupyter notebook.')
    nb_parser.add_argument('--foreground', action='store_true', default=False,
        help="Run jupyter notebook in the foreground.")

    # Parent parser for commands supporting expose.
    #expose_parser = argparse.ArgumentParser(add_help=False)
    #expose_parser.add_argument("-e", '--expose', default=False, action="store_true",
    #        help='Execute robot.expose to produce a pre-defined list of matplotlib figures.')
    #expose_parser.add_argument("-s", "--slide-mode", default=False, action="store_true",
    #        help="Used if --expose to iterate over figures. Expose all figures at once if not given on the CLI.")
    #expose_parser.add_argument("-t", "--slide-timeout", type=int, default=None,
    #        help="Close figure after slide-timeout seconds (only if slide-mode). Block if not specified.")

    # Build the main parser.
    parser = argparse.ArgumentParser(epilog=get_epilog() if with_epilog else "",
                                     formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('-V', '--version', action='version', version=abilab.__version__)

    # Create the parsers for the sub-command
    subparsers = parser.add_subparsers(dest='command', help='sub-command help', description="Valid subcommands")

    def add_args(p, *args):
        """Add arguments to subparser `p`."""
        for arg in args:
            if arg == "xmgrace":
                p.add_argument('--xmgrace', default=False, action="store_true",
                    help="Print bands in xmgrace format to stdout and exit.")
            elif arg == "bxsf":
                p.add_argument('--bxsf', default=False, action="store_true",
                    help=("Generate BXSF file suitable for the visualization of isosurfaces with Xcrysden"
                          "(xcrysden --bxsf FILE).\n Requires k-points in IBZ. Print to stdout and exit."))
            elif arg == "phononweb":
                p.add_argument("-web", "--phononwebsite", default=False, action="store_true",
                    help=("Visualize phonon band structure on the phononwebsite. "
                          "http://henriquemiranda.github.io/phononwebsite/"))
            elif arg == "browser":
                p.add_argument("-b", "--browser", default=None,
                    help="Define browser used by python webbrowser. "
                         "See https://docs.python.org/2/library/webbrowser.html#webbrowser.register")
            elif arg == "force":
                p.add_argument("-f", '--force', default=False, action="store_true",
                    help="Overwrite pre-existent files without prompting for confirmation.")
            else:
                raise ValueError("Invalid arg: %s" % arg)

    # Subparser for structure command.
    p_structure = subparsers.add_parser('structure', parents=[copts_parser], help=abiview_structure.__doc__)
    p_structure.add_argument("-a", "--appname", nargs="?", type=str, default="vesta",
        help=("Application name. Default: vesta. "
              "Possible options: %s, mayavi, vtk" % ", ".join(Visualizer.all_visunames())))

    # Subparser for hist command.
    p_hist = subparsers.add_parser('hist', parents=[copts_parser], help=abiview_hist.__doc__)
    p_hist.add_argument("-a", "--appname", nargs="?", default=None, const="ovito",
        help=("Application name. Default: ovito. "
              "Possible options: `%s`, `mpl` (matplotlib) `mayavi`, `vtk`" % ", ".join(Visualizer.all_visunames())))
    p_hist.add_argument("--xdatcar", default=False, action="store_true", help="Convert HIST file into XDATCAR format.")
    p_hist.add_argument("--to-unit-cell", default=False, action="store_true",
            help="Whether to translate sites into the unit cell.")
    add_args(p_hist, "force")

    # Subparser for data command.
    p_data = subparsers.add_parser('data', parents=[copts_parser], help=abiview_data.__doc__)
    p_data.add_argument("-i", "--use-index", default=False, action="store_true",
        help="Use the row index as x-value in the plot. By default the plotter uses the first column as x-values")

    # Subparser for abo command.
    #p_abo = subparsers.add_parser('abo', parents=[copts_parser], help=abiview_abo.__doc__)

    # Subparser for log command.
    p_dirviz = subparsers.add_parser('dirviz', parents=[copts_parser], help=abiview_dirviz.__doc__)
    p_dirviz.add_argument("-e", "--engine", type=str, default="fdp",
        help=("graphviz engine: ['dot', 'neato', 'twopi', 'circo', 'fdp', 'sfdp', 'patchwork', 'osage']. "
            "See http://www.graphviz.org/pdf/dot.1.pdf "
            "Use `conda install python-graphviz` or `pip install graphviz` to install the python package"))

    # Subparser for ebands command.
    p_ebands = subparsers.add_parser('ebands', parents=[copts_parser, slide_parser], help=abiview_ebands.__doc__)
    add_args(p_ebands, "xmgrace", "bxsf", "force")

    # Subparser for ebands command.
    p_skw = subparsers.add_parser('skw', parents=[copts_parser], help=abiview_skw.__doc__)
    p_skw.add_argument("-lp", "--lpratio", type=int, default=5,
        help=("Ratio between the number of star functions and the number of ab-initio k-points. "
              "The default should be OK in many systems, larger values may be required for accurate derivatives."))
    p_skw.add_argument("-ld", "--line-density", type=int, default=20,
        help ="Number of points in the smallest segment of the k-path.")

    # Subparser for fs command.
    p_fs = subparsers.add_parser('fs', parents=[copts_parser], help=abiview_fs.__doc__)
    p_fs.add_argument("-a", "--appname", type=str, default="mpl",
        help="Application name. Possible options: mpl (matplotlib, default), xsf (xcrysden), mayavi.")

    # Subparser for ddb command.
    p_ddb = subparsers.add_parser('ddb', parents=[copts_parser, slide_parser], help=abiview_ddb.__doc__)
    add_args(p_ddb, "xmgrace", "phononweb", "browser", "force")

    # Subparser for phbands command.
    p_phbands = subparsers.add_parser('phbands', parents=[copts_parser, slide_parser], help=abiview_phbands.__doc__)
    add_args(p_phbands, "xmgrace", "phononweb", "browser", "force")

    # Subparser for lobster command.
    p_lobster = subparsers.add_parser('lobster', parents=[copts_parser, ipy_parser, nb_parser],
        help=abiview_lobster.__doc__)
    p_lobster.add_argument("--prefix", type=str, default="", help="Prefix for lobster output files. Default: ''")

    # Subparser for denpot command.
    #p_denpot = subparsers.add_parser('denpot', parents=[copts_parser], help=abiview_denpot.__doc__)
    #p_denpot.add_argument("-a", "--appname", nargs="?", default=None, const="vesta",
    #        help=("Application name. Default: vesta. "
    #              "Possible options: `%s`, `mayavi`, `vtk`" % ", ".join(Visualizer.all_visunames())))
    #p_denpot.add_argument("--chgcar", default=False, action="store_true", "Convert Density to CHGCAR format.")
    #p_denpot.add_argument("--cube", default=False, action="store_true", "Convert Density/Potential to CUBE format.")

    return parser


@prof_main
def main():

    def show_examples_and_exit(err_msg=None, error_code=1):
        """Display the usage of the script."""
        sys.stderr.write(get_epilog())
        if err_msg:
            sys.stderr.write("Fatal Error\n" + err_msg + "\n")
        sys.exit(error_code)

    parser = get_parser(with_epilog=True)

    # Parse the command line.
    try:
        options = parser.parse_args()
    except Exception:
        show_examples_and_exit(error_code=1)

    if not options.command:
        show_examples_and_exit(error_code=1)

    # loglevel is bound to the string value obtained from the command line argument.
    # Convert to upper case to allow the user to specify --loglevel=DEBUG or --loglevel=debug
    import logging
    numeric_level = getattr(logging, options.loglevel.upper(), None)
    if not isinstance(numeric_level, int):
        raise ValueError('Invalid log level: %s' % options.loglevel)
    logging.basicConfig(level=numeric_level)

    if options.verbose > 2:
        print(options)

    if options.mpl_backend is not None:
        # Set matplotlib backend
        import matplotlib
        matplotlib.use(options.mpl_backend)

    if options.seaborn:
        # Use seaborn settings.
        import seaborn as sns
        sns.set(context=options.seaborn, style='darkgrid', palette='deep',
                font='sans-serif', font_scale=1, color_codes=False, rc=None)

    # Dispatch
    return globals()["abiview_" + options.command](options)

if __name__ == "__main__":
    sys.exit(main())



# coding: utf-8
r"""
Widgets for Pandas Dataframes based on seaborn API.

API reference

Distribution plots
jointplot(x, y[, data, kind, stat_func, ...])	Draw a plot of two variables with bivariate and univariate graphs.
pairplot(data[, hue, hue_order, palette, ...])	Plot pairwise relationships in a dataset.
distplot(a[, bins, hist, kde, rug, fit, ...])	Flexibly plot a univariate distribution of observations.
kdeplot(data[, data2, shade, vertical, ...])	Fit and plot a univariate or bivariate kernel density estimate.
rugplot(a[, height, axis, ax])	Plot datapoints in an array as sticks on an axis.

Regression plots
lmplot(x, y, data[, hue, col, row, palette, ...])	Plot data and regression model fits across a FacetGrid.
regplot(x, y[, data, x_estimator, x_bins, ...])	Plot data and a linear regression model fit.
residplot(x, y[, data, lowess, x_partial, ...])	Plot the residuals of a linear regression.
coefplot(formula, data[, groupby, ...])	Plot the coefficients from a linear model.

# Categorical plots
factorplot([x, y, hue, data, row, col, ...])	Draw a categorical plot onto a FacetGrid.
boxplot([x, y, hue, data, order, hue_order, ...])	Draw a box plot to show distributions with respect to categories.
violinplot([x, y, hue, data, order, ...])	Draw a combination of boxplot and kernel density estimate.
stripplot([x, y, hue, data, order, ...])	Draw a scatterplot where one variable is categorical.
swarmplot([x, y, hue, data, order, ...])	Draw a categorical scatterplot with non-overlapping points.
pointplot([x, y, hue, data, order, ...])	Show point estimates and confidence intervals using scatter plot glyphs.
barplot([x, y, hue, data, order, hue_order, ...])	Show point estimates and confidence intervals as rectangular bars.
countplot([x, y, hue, data, order, ...])	Show the counts of observations in each categorical bin using bars.

Matrix plots
heatmap(data[, vmin, vmax, cmap, center, ...])	Plot rectangular data as a color-encoded matrix.
clustermap(data[, pivot_kws, method, ...])	Plot a hierarchically clustered heatmap of a pandas DataFrame

Timeseries plots
tsplot(data[, time, unit, condition, value, ...])	Plot one or more timeseries with flexible representation of uncertainty.

Miscellaneous plots
palplot(pal[, size])	Plot the values in a color palette as a horizontal array.

Axis grids
FacetGrid(data[, row, col, hue, col_wrap, ...])	Subplot grid for plotting conditional relationships.
PairGrid(data[, hue, hue_order, palette, ...])	Subplot grid for plotting pairwise relationships in a dataset.
JointGrid(x, y[, data, size, ratio, space, ...])	Grid for drawing a bivariate plot with marginal univariate plots.
"""
from __future__ import print_function, division, unicode_literals, absolute_import

import sys
import ipywidgets as ipw
import seaborn as sns
import abipy.display.utils as ut

from functools import wraps
from collections import OrderedDict
from IPython.display import display, clear_output

__all__ = [
    #"api_selector",
    # Distribution plots
    "jointplot",
    "pairplot",
    #"distplot",
    #"kdeplot",
    #"rugplot",
    # Regression plots
    "lmplot",
    #"regplot",
    #"residplot",
    #"coefplot",
    # Categorical plots
    "factorplot",
    "boxplot",
    "violinplot",
    "stripplot",
    "swarmplot",
    "pointplot",
    "barplot",
    "countplot",
    # Matrix plots
    #"heatmap",
    #"clustermap",
    # Timeseries plots
    #"tsplot",
    # Miscellaneous plots
    #"palplot",
]


#def api_selector(data, funcname="countplot"):
#    """
#    A widgets with ToogleButtons that allow the user to select and display
#    the widget associated to the different seaborn functions.
#    """
#    this_module = sys.modules[__name__]
#    name2wfunc = OrderedDict()
#    for a in __all__:
#        if a == "api_selector": continue
#        func = this_module.__dict__.get(a)
#        if not callable(func): continue
#        name2wfunc[func.__name__] = func
#
#    w1 = ipw.ToggleButtons(description='seaborn API', options=list(name2wfunc.keys()))
#    w1.value = funcname
#    w2 = name2wfunc[funcname](data)
#    box = ipw.VBox(children=[w1, w2])
#
#    def on_value_change(change):
#        #print(change)
#        box.close()
#        clear_output()
#        api_selector(data, funcname=change["new"])
#    w1.observe(on_value_change, names='value')
#
#    return display(box)

#def _help_button(funcname):
#    btn = ipw.Button(description="Help", button_style='info', tooltip='Click me', icon='check')
#    btn.value = "help"
#    def func(btn):
#        open_seaborn_doc(funcname)
#    btn.on_click(func)
#    return btn
#
#
#def open_seaborn_doc(funcname):
#    # http://seaborn.pydata.org/generated/seaborn.pairplot.html#seaborn.pairplot"
#    url = "http://seaborn.pydata.org/generated/seaborn.%s.html#seaborn.%s" % (funcname, funcname)
#    import webbrowser
#    try:
#        return webbrowser.open(url)
#    except webbrowser.Error as exc:
#        # Warn the user and ignore the exception.
#        import warnings
#        warnings.warn(str(exc) + "\nwhile trying to open url: %s" % url)


@wraps(sns.jointplot)
def jointplot(data, joint_kws=None, marginal_kws=None, annot_kws=None, **kwargs):

    def sns_joinplot(x, y, kind, color):  # pragma: no cover
        x, y, color = ut.widget2py(x, y, color)
        # TODO: stat_func
        return sns.jointplot(x, y, data=data, kind=kind, # stat_func=<function pearsonr>,
                            color=color, size=6, ratio=5, space=0.2, dropna=True, xlim=None, ylim=None,
                            joint_kws=joint_kws, marginal_kws=marginal_kws, annot_kws=annot_kws, **kwargs)

    allcols = ["None"] + list(data.keys())
    return ipw.interact_manual(
                sns_joinplot,
                x=allcols,
                y=allcols,
                kind=["scatter", "reg", "resid", "kde", "hex"],
                color=ut.colors_dropdow(),
                #button=_help_button("joinplot"),
            )


@wraps(sns.pairplot)
def pairplot(data, plot_kws=None, diag_kws=None, grid_kws=None):
    # TODO: Write widget with multiple checkboxes to implement lists.

    def sns_pairplot(x_vars, y_vars, hue, kind, diag_kind): # pragma: no cover
        x_vars, y_vars, hue = ut.widget2py(x_vars, y_vars, hue)
        return sns.pairplot(data, hue=hue, hue_order=None, palette=None, vars=None, x_vars=x_vars, y_vars=y_vars,
                     kind=kind, diag_kind=diag_kind, markers=None, size=2.5, aspect=1, dropna=True,
                     plot_kws=plot_kws, diag_kws=diag_kws, grid_kws=grid_kws)

    allcols = ["None"] + list(data.keys())
    return ipw.interact_manual(
                sns_pairplot,
                x_vars=allcols,
                y_vars=allcols,
                hue=allcols,
                kind=["scatter", "ref"],
                diag_kind=["hist", "kde"],
            )


"""
@wraps(sns.distplot)
def distplot(data, fit=None, hist_kws=None, kde_kws=None, rug_kws=None, fit_kws=None):

    def sns_distplot(hist, kde, rug, color, vertical, norm_hist):
        color = ut.widget2py(color)
        ax, fig, _ = ut.get_ax_fig_plt()
        return sns.distplot(a, bins=None, hist=hist, kde=kde, rug=rug, fit=fit,
                     hist_kws=hist_kws, kde_kws=kde_kws, rug_kws=rug_kws, fit_kws=fit_kws,
                     color=clor, vertical=vertical, norm_hist=norm_hist, axlabel=None, label=None, ax=ax)

    allcols = ["None"] + list(data.keys())
    return ipw.interact_manual(
                sns_distplot,
                hist=True,
                kde=True,
                rug=False,
                color=ut.colors_dropdow(),
                vertical=False,
                norm_hist=False,
            )


@wraps(sns.kdeplot)
def kdeplot(data, **kwargs):

    def sns_kdeplot()
        color = ut.widget2py(color)
        ax, fig, _ = ut.get_ax_fig_plt()

        return sns.kdeplot(data, data2=None, shade=False, vertical=False, kernel='gau', bw='scott',
                    gridsize=100, cut=3, clip=None, legend=True, cumulative=False, shade_lowest=True, ax=ax, **kwargs)

    allcols = ["None"] + list(data.keys())
    return ipw.interact_manual(
                sns_kdeplot,
                color=ut.colors_dropdow(),
            )
"""

####################
# Regression plots #
####################

@wraps(sns.lmplot)
def lmplot(data, scatter_kws=None, line_kws=None):

    def sns_lmplot(x, y, hue, col, row, legend, size):  # pragma: no cover
        x, y, hue, col, row = ut.widget2py(x, y, hue, col, row)

        return sns.lmplot(x, y, data, hue=hue, col=col, row=row, palette=None, col_wrap=None,
                   size=size, aspect=1, markers='o', sharex=True, sharey=True, hue_order=None,
                   col_order=None, row_order=None, legend=legend, legend_out=True,
                   x_estimator=None, x_bins=None, x_ci='ci', scatter=True, fit_reg=True,
                   ci=95, n_boot=1000, units=None, order=1, logistic=False, lowess=False, robust=False,
                   logx=False, x_partial=None, y_partial=None, truncate=False, x_jitter=None, y_jitter=None,
                   scatter_kws=scatter_kws, line_kws=line_kws)

    allcols = ["None"] + list(data.keys())
    return ipw.interact_manual(
                sns_lmplot,
                x=allcols,
                y=allcols,
                hue=allcols,
                col=allcols,
                row=allcols,
                legend=True,
                size=ut.size_slider(default=5),
            )


#@wraps(sns.interactplot)
#def interactplot(data, contour_kws=None, scatter_kws=None, **kwargs):
#
#    def sns_interactplot(x1, x2, y, filled, colorbar, logistic):
#        ax, fig, _ = ut.get_ax_fig_plt()
#        return sns.interactplot(x1, x2, y, data=data, filled=filled, cmap='RdBu_r', colorbar=colorbar,
#                         levels=30, logistic=logistic, contour_kws=contour_kws, scatter_kws=scatter_kws,
#                         ax=ax, **kwargs)
#
#    allcols = list(data.keys())
#    return ipw.interact_manual(
#                sns_interactplot,
#                x1=allcols,
#                x2=allcols,
#                y=allcols,
#                filled=False,
#                colorbar=True,
#                logistic=False,
#            )


#####################
# Categorical plots #
#####################

@wraps(sns.factorplot)
def factorplot(data, facet_kws=None, **kwargs):

    def sns_factorplot(x, y, hue, color, kind, size, legend):  # pragma: no cover
        x, y, hue, color = ut.widget2py(x, y, hue, color)
        return sns.factorplot(x=x, y=y, hue=hue, data=data, row=None, col=None, col_wrap=None, # estimator=<function mean>,
                       ci=95, n_boot=1000, units=None, order=None, hue_order=None, row_order=None, col_order=None,
                       kind=kind, size=size, aspect=1, orient=None, color=color, palette=None,
                       legend=legend, legend_out=True, sharex=True, sharey=True, margin_titles=False,
                       facet_kws=facet_kws, **kwargs)

    allcols = ["None"] + list(data.keys())
    return ipw.interact_manual(
                sns_factorplot,
                x=allcols,
                y=allcols,
                hue=allcols,
                color=ut.colors_dropdow(),
                kind=["point", "bar", "count", "box", "violin", "strip"],
                size=ut.size_slider(default=4),
                legend=True,
            )


@wraps(sns.boxplot)
def boxplot(data, **kwargs):

    def sns_boxplot(x, y, hue, orient, color, saturation, notch): # pragma: no cover
        x, y, hue, orient, color = ut.widget2py(x, y, hue, orient, color)
        ax, fig, _ = ut.get_ax_fig_plt()
        return sns.boxplot(x=x, y=y, hue=hue, data=data, order=None, hue_order=None, orient=orient,
                          color=color, palette=None, saturation=saturation, width=0.8, fliersize=5, linewidth=None,
                          whis=1.5, notch=notch, ax=ax, **kwargs)

    allcols = ["None"] + list(data.keys())
    return ipw.interact_manual(
                sns_boxplot,
                x=allcols,
                y=allcols,
                hue=allcols,
                orient=["None", "v", "h"],
                color=ut.colors_dropdow(),
                saturation=ut.saturation_slider(default=0.75),
                notch=False,
            )


@wraps(sns.violinplot)
def violinplot(data, **kwargs):

    def sns_violinplot(x, y, hue, bw, scale, inner, split, orient, color, saturation): # pragma: no cover
        x, y, hue, inner, orient, color = ut.widget2py(x, y, hue, inner, orient, color)
        ax, fig, _ = ut.get_ax_fig_plt()

        sns.violinplot(x=x, y=y, hue=hue, data=data, order=None, hue_order=None,
                       bw=bw, cut=2, scale=scale, scale_hue=True,
                       gridsize=100, width=0.8, inner=inner, split=split, orient=orient,
                       linewidth=None, color=color, palette=None, saturation=saturation, ax=ax, **kwargs)

    allcols = ["None"] + list(data.keys())
    return ipw.interact_manual(
                sns_violinplot,
                x=allcols,
                y=allcols,
                hue=allcols,
                bw=["scott", "silverman", "float"],
                scale=["area", "count", "width"],
                inner=["box", "quartile", "point", "stick", "None"],
                split=False,
                orient=["None", "v", "h"],
                color=ut.colors_dropdow(),
                saturation=ut.saturation_slider(default=0.75),
            )


@wraps(sns.stripplot)
def stripplot(data, **kwargs):

    def sns_stripplot(x, y, hue, split, orient, color, size, linewidth): # pragma: no cover
        x, y, hue, orient, color = ut.widget2py(x, y, hue, orient, color)
        ax, fig, _ = ut.get_ax_fig_plt()
        return sns.stripplot(x=x, y=y, hue=hue, data=data, order=None, hue_order=None, jitter=False,
                            split=split, orient=orient, color=color, palette=None, size=size, edgecolor='gray',
                            linewidth=linewidth, ax=ax, **kwargs)

    allcols = ["None"] + list(data.keys())
    return ipw.interact_manual(
                sns_stripplot,
                x=allcols,
                y=allcols,
                hue=allcols,
                split=False,
                orient=["None", "v", "h"],
                color=ut.colors_dropdow(),
                size=ut.size_slider(default=5),
                linewidth=ut.linewidth_slider(default=0),
            )


@wraps(sns.swarmplot)
def swarmplot(data, **kwargs):

    def sns_swarmplot(x, y, hue, split, orient, color, size, linewidth): # pragma: no cover
        x, y, hue, orient, color = ut.widget2py(x, y, hue, orient, color)
        ax, fig, _ = ut.get_ax_fig_plt()
        return sns.swarmplot(x=x, y=y, hue=hue, data=data, order=None, hue_order=None,
                            split=split, orient=orient, color=color, palette=None, size=size,
                            edgecolor='gray', linewidth=linewidth, ax=ax, **kwargs)

    allcols = ["None"] + list(data.keys())
    return ipw.interact_manual(
                sns_swarmplot,
                x=allcols,
                y=allcols,
                hue=allcols,
                split=False,
                orient=["None", "v", "h"],
                color=ut.colors_dropdow(),
                size=ut.size_slider(default=5),
                linewidth=ut.linewidth_slider(default=0),
            )


@wraps(sns.pointplot)
def pointplot(data, **kwargs):

    def sns_pointplot(x, y, hue, split, join, orient, color, linewidth): # pragma: no cover
        x, y, hue, orient, color = ut.widget2py(x, y, hue, orient, color)
        ax, fig, _ = ut.get_ax_fig_plt()
        return sns.pointplot(x=x, y=y, hue=hue, data=data, order=None, hue_order=None, # estimator=<function mean>,
                            ci=95, n_boot=1000, units=None, markers='o', linestyles='-', dodge=False, join=join, scale=1,
                            orient=orient, color=color, palette=None, ax=ax, errwidth=None, capsize=None, **kwargs)

    allcols = ["None"] + list(data.keys())
    return ipw.interact_manual(
                sns_pointplot,
                x=allcols,
                y=allcols,
                hue=allcols,
                split=False,
                join=True,
                orient=["None", "v", "h"],
                color=ut.colors_dropdow(),
                linewidth=ut.linewidth_slider(default=0),
            )


@wraps(sns.barplot)
def barplot(data, **kwargs):

    def sns_barplot(x, y, hue, orient, color, saturation): # pragma: no cover
        x, y, hue, orient, color = ut.widget2py(x, y, hue, orient, color)
        ax, fig, _ = ut.get_ax_fig_plt()
        return sns.barplot(x=x, y=y, hue=hue, data=data, order=None, hue_order=None, # estimator=<function mean>,
                           ci=95, n_boot=1000, units=None, orient=orient, color=color, palette=None,
                           saturation=saturation, errcolor='.26', ax=ax, **kwargs) # errwidth=None, capsize=None, # New args added in ??

    allcols = ["None"] + list(data.keys())
    return ipw.interact_manual(
                sns_barplot,
                x=allcols,
                y=allcols,
                hue=allcols,
                orient=["None", "v", "h"],
                color=ut.colors_dropdow(),
                saturation=ut.saturation_slider(default=0.75),
            )


@wraps(sns.countplot)
def countplot(data, **kwargs):

    def sns_countplot(x, y, hue, color, saturation): # pragma: no cover
        x, y, hue, color = ut.widget2py(x, y, hue, color)
        ax, fig, _ = ut.get_ax_fig_plt()
        return sns.countplot(x=x, y=y, hue=hue, data=data, order=None, hue_order=None, orient=None,
                             color=color, palette=None, saturation=saturation, ax=ax, **kwargs)

    allcols = ["None"] + list(data.keys())
    return ipw.interact_manual(
                sns_countplot,
                x=allcols,
                y=allcols,
                hue=allcols,
                color=ut.colors_dropdow(),
                saturation=ut.saturation_slider(default=0.75),
            )

################
# Matrix plots #
################

@wraps(sns.heatmap)
def heatmap(data, annot_kws=None, cbar_kws=None, **kwargs):

    def sns_heatmap(): # pragma: no cover
        ax, fig, _ = ut.get_ax_fig_plt()
        return sns.heatmap(data, vmin=None, vmax=None, cmap=None, center=None, robust=False, annot=None,
                           fmt='.2g', annot_kws=annot_kws, linewidths=0, linecolor='white', cbar=True,
                           cbar_kws=cbar_kws, cbar_ax=None, square=False, ax=ax,
                           xticklabels=True, yticklabels=True, mask=None, **kwargs)

    return ipw.interact_manual(
                sns_heatmap,
            )


@wraps(sns.clustermap)
def clustermap(data, pivot_kws=None, cbar_kws=None, **kwargs):

    def sns_clustermap(): # pragma: no cover
        return sns.clustermap(data, pivot_kws=pivot_kws, method='average', metric='euclidean',
                              z_score=None, standard_scale=None, figsize=None, cbar_kws=cbar_kws,
                              row_cluster=True, col_cluster=True, row_linkage=None, col_linkage=None,
                              row_colors=None, col_colors=None, mask=None, **kwargs)

    return ipw.interact_manual(
                sns_clustermap,
            )



#!/usr/bin/env python
r"""
Starting a Flow from external files
===================================

This example shows how to build a flow for Nscf calculations
in which the dependency is given by an external density file.
"""
from __future__ import print_function, division, unicode_literals, absolute_import

import sys
import os
import abipy.data as abidata
import abipy.abilab as abilab
import abipy.flowtk as flowtk


def make_scf_nscf_inputs(paral_kgb=1):
    """Returns two input files: GS run and NSCF on a high symmetry k-mesh."""
    multi = abilab.MultiDataset(structure=abidata.cif_file("si.cif"),
                                pseudos=abidata.pseudos("14si.pspnc"), ndtset=2)

    # Global variables
    ecut = 6
    global_vars = dict(
        ecut=ecut,
        nband=8,
        nstep=15,
        paral_kgb=paral_kgb,
    )

    if multi.ispaw:
        global_vars.update(pawecutdg=2*ecut)

    multi.set_vars(global_vars)

    # Dataset 1 (GS run)
    multi[0].set_kmesh(ngkpt=[8, 8, 8], shiftk=[0, 0, 0])
    multi[0].set_vars(tolvrs=1e-6)

    # Dataset 2 (NSCF run)
    kptbounds = [
        [0.5, 0.0, 0.0], # L point
        [0.0, 0.0, 0.0], # Gamma point
        [0.0, 0.5, 0.5], # X point
    ]

    multi[1].set_kpath(ndivsm=6, kptbounds=kptbounds)
    multi[1].set_vars(tolwfr=1e-12)

    # Generate two input files for the GS and the NSCF run
    scf_input, nscf_input = multi.split_datasets()

    return scf_input, nscf_input


def build_flow(options):
    # Working directory (default is the name of the script with '.py' removed and "run_" replaced by "flow_")
    if not options.workdir:
        options.workdir = os.path.basename(__file__).replace(".py", "").replace("run_", "flow_")

    # Get the SCF and the NSCF input.
    scf_input, nscf_input = make_scf_nscf_inputs()

    # Build the flow.
    flow = flowtk.Flow(options.workdir, manager=options.manager)

    # Create a Work, all tasks in work will start from the DEN file.
    # Note that the file must exist when the work is created
    # Use the standard approach based on tasks and works if
    # there's a node who needs a file produced in the future.
    work = flowtk.Work()
    den_filepath = abidata.ref_file("si_DEN.nc")
    work.register_nscf_task(nscf_input, deps={den_filepath: "DEN"})
    flow.register_work(work)

    return flow


# This block generates the thumbnails in the Abipy gallery.
# You can safely REMOVE this part if you are using this script for production runs.
if os.getenv("READTHEDOCS", False):
    __name__ = None
    import tempfile
    options = flowtk.build_flow_main_parser().parse_args(["-w", tempfile.mkdtemp()])
    #build_flow(options).plot_networkx(tight_layout=True)
    build_flow(options).graphviz_imshow()


@flowtk.flow_main
def main(options):
    """
    This is our main function that will be invoked by the script.
    flow_main is a decorator implementing the command line interface.
    Command line args are stored in `options`.
    """
    return build_flow(options)


if __name__ == "__main__":
    sys.exit(main())



"""Integration tests for BSE flows."""
from __future__ import print_function, division, unicode_literals, absolute_import

import pytest
import abipy.data as abidata
import abipy.abilab as abilab
import abipy.flowtk as flowtk

from abipy.core.testing import has_abinit, has_matplotlib

# TODO
#def itest_bse_with_mdf(fwp, tvars):
#    pseudos = abidata.pseudos("14si.pspnc")
#    structure = abilab.Structure.from_file(abidata.cif_file("si.cif"))
#
#    kppa = scf_kppa = 1
#    nscf_nband = 6
#    nscf_ngkpt = [4,4,4]
#    nscf_shiftk = [0.1, 0.2, 0.3]
#    bs_loband = 2
#    bs_nband = nscf_nband
#    soenergy = 0.7
#    mdf_epsinf = 12
#    max_ncpus = 1
#    ecuteps = 2
#
#    extra_abivars = dict(
#        ecut=12,
#        istwfk="*1",
#    )
#
#    flow = flowtk.Flow(workdir=fwp.workdir, manager=fwp.manager)
#
#    # BSE calculation with model dielectric function.
#    from pymatgen.io.abinit.calculations import bse_with_mdf
#    work = bse_with_mdf(structure, pseudos, scf_kppa, nscf_nband, nscf_ngkpt, nscf_shiftk,
#                       ecuteps, bs_loband, bs_nband, soenergy, mdf_epsinf,
#                       accuracy="normal", spin_mode="unpolarized", smearing=None,
#                       charge=0.0, scf_solver=None, **extra_abivars)
#
#    flow.register_work(work)
#    flow.allocate()
#    flow.build_and_pickle_dump(abivalidate=True)
#
#    for task in flow:
#        task.start_and_wait()
#        assert task.status == task.S_DONE
#
#    flow.check_status()
#    flow.show_status()
#    assert all(work.finalized for work in flow)
#    assert flow.all_ok
#    assert flow.validate_json_schema()



#!/usr/bin/env python
import wx

from abipy.gui.eos import EosFrame

volumes = [13.72, 14.83, 16.0, 17.23, 18.52]
energies = [-56.29, -56.41, -56.46, -56.46, -56.42]

app = wx.App()
frame = EosFrame(None, volumes, energies)
frame.Show()
app.SetTopWindow(frame)
app.MainLoop()
